/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance.classconcepts;
import java.util.HashSet;
import java.util.Set;


/**
 *Reprsente une classe
 * @author OUEDRAOGO Moussa
 * @since 25-01-2007
 *
 */
public class Classe {
	/**
	 * Le id de la classe	 
	 */
	private String id;
	
	/**
	 * Le nom de la classe
	 */
	private String nom;
	
	/**
	 * La visibilite de la classe (public, private, protected, etc)	 
	 */
	private String visibilite;
	
	/**
	 * Les diffrents attributs de la classe
	 */
	private Set<SimpleProperty> proprietesSimples=new HashSet<SimpleProperty>();
	
	/**
	 * Permet de savoir  si on a une classe association ou non	 
	 */
	private boolean estClasseAssociation=false;
	
	/**
	 * Permet de savoir si la classe est fille d'une autre classe
	 */
	private boolean child;
	
	/**
	 * Les diffrentes associations de la classe
	 */ 
	private Set<Association> associations=new HashSet<Association>();
	
	/**
	 * Cette mthode retourne les diffrents proprits simples de la classe (proprits de type chaine, entire, dcimal, date)
	 * @return les diffrents proprits simples de la classe (proprits de type chaine, entire, dcimal, date)
	 */
	public Set<SimpleProperty> getProprietesSimples() {
		return proprietesSimples;
	}

	/**
	 * Le package dans lequel se trouve cette classe
	 */
	private String classPackage;
	/**
	 * Cette mthode permet de modifier la liste des proprits simples de la classe
	 * @param attributs
	 */
	public void setProprietesSimples(Set<SimpleProperty> attributs) {
		this.proprietesSimples = attributs;
	}

	/**
	 * Cette mthode retourne le id de la classe
	 * @return le id de la classe
	 */
	public String getId() {
		return id;
	}

	/**
	 * Cette mthode modifie le id de la classe
	 * @param id le nouvel id de la classe
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 *Cette mthode retourne le nom de la classe 
	 * @return nom de la classe
	 */
	public String getNom() {
		return nom;
	}

	/**
	 * Cette mthode modifie le nom de la classe
	 * @param nom le nouveau nom de la classe
	 */
	public void setNom(String nom) {
		this.nom = nom;
	}

	/**
	 * Cette mthode retourne la visisibilit de la classe
	 * @return la visisibilit de la classe
	 */
	public String getVisibilite() {
		return visibilite;
	}

	/**
	 * Cette mthode modifie la visibilit de la classe
	 * @param visibilite
	 */
	public void setVisibilite(String visibilite) {
		this.visibilite = visibilite;
	}

	/**
	 * Cette mthode retourne vraie si la classe est une classe-association et faux sinon
	 * @return vraie si la classe est une classe-association et faux sinon
	 */
	public boolean isEstClasseAssociation() {
		return estClasseAssociation;
	}

	/**
	 * Cette mthode modifie le fait qu'une classe est classe-association ou non
	 * @param estClasseAssociation le nouvel statut classe-association
	 */
	public void setEstClasseAssociation(boolean estClasseAssociation) {
		this.estClasseAssociation = estClasseAssociation;
	}

	/**
	 * Cette mthode retourne vraie si la classe est fille et faux sinon
	 * @return vraie si la classe est fille et faux sinon
	 */
	public boolean isChild() {
		return child;
	}

	/**
	 * Cette mthode modifie le fait qu'une classe est fille ou non
	 * @param child le nouvel statut child
	 */
	public void setChild(boolean child) {
		this.child = child;
	}

	/**
	 * Cette mthode retourne la liste des diffrentes associations de la classe
	 * @return la liste des diffrentes associations de la classe
	 */
	public Set<Association> getAssociations() {
		return associations;
	}

	/**
	 * Cette mthode modifie la liste des diffrentes associations de la classe
	 * @param associations la liste des diffrentes associations de la classe
	 */
	public void setAssociations(Set<Association> associations) {
		this.associations = associations;
	}

	public String getClassPackage() {
		return classPackage;
	}

	public void setClassPackage(String classPackage) {
		this.classPackage = classPackage;
	}
	
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if ((obj == null) || (obj.getClass() != this.getClass()))
			return false;
		// object must be Test at this point
		Classe classTemp= (Classe) obj;
		return this.getId().compareTo(classTemp.getId())==0;
	}

	public int hashCode() {
		int hash = 7;		
		hash = 31 * hash + (null == this.getId() ? 0 : this.getId().hashCode());
		return hash;
	}
	
	
	

	
	
}
