/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance.classconcepts;

import java.util.Enumeration;
import java.util.Hashtable;



/**
 * La classe InfosModele encapsule l'ensemble des ressources pour la 
 * gnration des diffrentes classes et fichiers
 * @author OUEDRAOGO Moussa
 * @since 25-01-2007
 *
 */
public class InfosModel {
	Hashtable<String,Classe>  classes;
	Hashtable <String,Association> associations;
	Hashtable<String,String> types;
	
	/**
	 * Cette mthode retourne la classe dont le nom est pass
	 * en paramtre. Cette methode retourne null si aucune classe
	 * n'est trouve
	 * @param nomClasse le nom de la classe  rechercher
	 * @return la classe dont le nom est pass
	 * en paramtre
	 */
	public Classe findClasse(String nomClasse){
		Enumeration<Classe>enumeration=classes.elements();
		while(enumeration.hasMoreElements()){
			Classe classe=enumeration.nextElement();
			if (classe.getNom().compareTo(nomClasse)==0)
				return classe;
		}
		return null;
	}
	
	/**
	 * Cette mthode retourne les diffrentes classes
	 * @return les diffrentes classes
	 */
	public Hashtable<String,Classe> getClasses() {
		return classes;
	}
	/**
	 * Cette mthode permet de modifier la liste des classes
	 * @param classes la nouvelle liste
	 */
	public void setClasses(Hashtable<String,Classe> classes) {
		this.classes = classes;
	}
	
	/**
	 * Cette mthode retourne les diffrentes Associations
	 * @return les diffrentes associations
	 */
	public Hashtable<String,Association> getAssociations() {
		return associations;
	}
	
	/**
	 * Cette mthode permet de modifier la liste des associations
	 * @param associations la nouvelle liste
	 */
	public void setAssociations(Hashtable<String,Association> associations) {
		this.associations = associations;
	}
	
	/**
	 * Cette mthode retourne les diffrents types
	 * @return les diffrentes types
	 */
	public Hashtable<String,String> getTypes() {
		return types;
	}
	
	/**
	 * Cette mthode permet de modifier la liste des types
	 * @param types la nouvelle liste
	 */
	public void setTypes(Hashtable<String,String> types) {
		this.types = types;
	}
	
	
}
