/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance.classcreator;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;

import oued.persistance.classconcepts.Classe;
import oued.persistance.classconcepts.InfosModel;
import oued.persistance.utils.FileCopier;
import oued.persistance.utils.ULoader;
import oued.persistance.utils.UtilityClassFunction;

/**
 * La classe {@link ClassGenerator} permet de crer les diffrents pojo et dao du projet
 * @author OUEDRAOGO Moussa
 * @since 30-01-2007
 */
public class ClassGenerator {
	private InfosModel infosModel;
	
	/**
	 * Constructeur personnalis de {@link ClassGenerator}
	 * @param infosModel le infosModele qui contient toutes les ressources pour la gnration
	 */
	public ClassGenerator(InfosModel infosModel){		
		this.infosModel=infosModel;
	}
	
	/**
	 * Cette mthode permet de gnrer l'ensemble des classes DAO et POJO
	 * @param genererDAO doit on gnrer les dao ou non
	 * @param genererPOJO doit on gnrer les pojo ou non
	 */
	public void produceInfosModelClasses(String workingDir,String srcDir,boolean genererDAO,boolean genererPOJO) {
		if (!genererDAO && !genererPOJO) return; //Il n'ya rien a generer
		
		Enumeration<Classe>  enumeration=infosModel.getClasses().elements();
		PhysicalClassCreator javaCreator=new PhysicalClassCreator(workingDir,srcDir);
		PhysicalClassCreator daoCreator=new PhysicalClassCreator(workingDir,srcDir);
		while(enumeration.hasMoreElements()){
			Classe classe=(Classe)enumeration.nextElement();
			DAOJavaClass daoContent=null;
			JavaClass javaContent=null;
			
			if (genererPOJO){
				javaContent=new JavaClass(classe,infosModel);
				javaContent.createContent();
			}
			
			if (genererDAO){
				daoContent=new DAOJavaClass(classe);
				daoContent.createContent();
			}
			if (genererPOJO){
				try {
					javaCreator.createClasse(javaContent);
				} catch (IOException e) {
					System.out.println("PersistanceZero: Echec de creation de la classe java "+UtilityClassFunction.firstLetterUpperCase(classe.getNom())+".java");
				}
			}
			if (genererDAO){
				if (daoContent.getContent()!=null){
					try {
						daoCreator.createClasse(daoContent);
					} catch (IOException e) {
						System.out.println("PersistanceZero: Echec de creation du dao DAO"+UtilityClassFunction.firstLetterUpperCase(classe.getNom())+".java");
					}
			}
			}
			
		}
		
	}
	
	
	
	/**
	 * Cette mthode permet de copier et de crer les diffrents fichiers ressources ncessaires au fonctionnement des classes
	 * gnres
	 */
	public void produceRessourcesClasse(String workingDir,String srcDir,String ressourcesPackage){
		/**
		 * recuperer les fichiers de ressources dans le jar
		 * 
		 */
		InputStream abstractDAO =ULoader.getFileInJar("oued/persistance/ressources/AbstractDAO.res");
		InputStream dataAccessLayer =ULoader.getFileInJar("oued/persistance/ressources/DataAccessLayerException.res");
		InputStream hibernateUtil =ULoader.getFileInJar("oued/persistance/ressources/HibernateUtil.res");
		InputStream gestionnaireTransaction =ULoader.getFileInJar("oued/persistance/ressources/TransactionManager.res");
		InputStream serviceLayerException =ULoader.getFileInJar("oued/persistance/ressources/ServiceLayerException.res");
		InputStream critere =ULoader.getFileInJar("oued/persistance/ressources/Critere.res");
		InputStream critereComplexe =ULoader.getFileInJar("oued/persistance/ressources/CritereComplexe.res");
		InputStream introspecteur =ULoader.getFileInJar("oued/persistance/ressources/Introspecteur.res");
		 
		 /**
		  * Copier et crer les fichiers de ressources. 
		  */
		String nomFichierDAO=workingDir+"\\"+srcDir+"\\"+UtilityClassFunction.replacePointBySlash(ressourcesPackage)+"\\AbstractDAO.java";
		File copieAsbstractDAO=new File(nomFichierDAO);
		File copieDataAccessLayer=new File(workingDir+"\\"+srcDir+"\\"+UtilityClassFunction.replacePointBySlash(ressourcesPackage)+"\\DataAccessLayerException.java");
		File copieHibernateUtil=new File(workingDir+"\\"+srcDir+"\\"+UtilityClassFunction.replacePointBySlash(ressourcesPackage)+"\\HibernateUtil.java");
		File copieGestionnaireTransaction=new File(workingDir+"\\"+srcDir+"\\"+UtilityClassFunction.replacePointBySlash(ressourcesPackage)+"\\TransactionManager.java");
		File copieServiceLayerException=new File(workingDir+"\\"+srcDir+"\\"+UtilityClassFunction.replacePointBySlash(ressourcesPackage)+"\\ServiceLayerException.java");
		File copieCritere=new File(workingDir+"\\"+srcDir+"\\"+UtilityClassFunction.replacePointBySlash(ressourcesPackage)+"\\Critere.java");
		File copieCritereCritere=new File(workingDir+"\\"+srcDir+"\\"+UtilityClassFunction.replacePointBySlash(ressourcesPackage)+"\\CritereComplexe.java");
		File copieIntrospecteur=new File(workingDir+"\\"+srcDir+"\\"+UtilityClassFunction.replacePointBySlash(ressourcesPackage)+"\\Introspecteur.java");
		 
		FileCopier fichierCopieur=new FileCopier();
		try {
		fichierCopieur.copyFile(abstractDAO, copieAsbstractDAO);
		fichierCopieur.copyFile(dataAccessLayer, copieDataAccessLayer);
		fichierCopieur.copyFile(hibernateUtil, copieHibernateUtil);
		fichierCopieur.copyFile(gestionnaireTransaction, copieGestionnaireTransaction);
		fichierCopieur.copyFile(serviceLayerException, copieServiceLayerException);
		fichierCopieur.copyFile(critere, copieCritere);
		fichierCopieur.copyFile(critereComplexe, copieCritereCritere);
		
		copieAsbstractDAO.createNewFile();
		copieDataAccessLayer.createNewFile();
		copieHibernateUtil.createNewFile();
		copieGestionnaireTransaction.createNewFile();
		copieServiceLayerException.createNewFile();
		copieCritere.createNewFile();
		copieCritereCritere.createNewFile();
			
		} catch (IOException e) {
			e.printStackTrace();
			System.out.println("PersistanceZero:Une erreur est survenue lors de la copie des ressources." +
					" Il se pourrait qu'une ou plusieurs ressource ne soient pas cres");
		}
	}
	
	
}
