/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance.classcreator;

import oued.persistance.classconcepts.Classe;
import oued.persistance.utils.UtilityClassFunction;

/**
 * Cette classe permet de crer le contenu d'un dao
 * 
 * @author OUEDRAOGO Moussa
 * @since 25-01-2007
 * 
 */
public class DAOJavaClass extends JavaClass {
	
	/***
	 * Constructeur personnalis
	 * @param classe la classe pour laquelle le contenu sera cr
	 */
	public DAOJavaClass(Classe classe) {
		super(classe, null);
		this.setNomClasse("DAO" + classe.getNom() + ".java");
	}

	/**
	 * Cette mthode permet de crer le contenu du dao
	 */
	public void createContent() {
		String content = "";
		if ((this.getClasse().isChild())
				|| (this.getClasse().isEstClasseAssociation())) {
			content = null;
			this.setContent(content);
			return;
		}

		content += "";
		content += MotCle.PACKAGE + MotCle.SPACE + this.getClasse().getClassPackage()
				+ MotCle.POINT_COMMA;

		content += MotCle.NEW_LINE;
		content += MotCle.NEW_LINE;

		content += MotCle.IMPORT + MotCle.SPACE + "java.util.*"
				+ MotCle.POINT_COMMA;
		content += MotCle.NEW_LINE;
		content += MotCle.IMPORT + MotCle.SPACE + "oued.ressources.*"
				+ MotCle.POINT_COMMA;

		content += MotCle.NEW_LINE;
		content += MotCle.NEW_LINE;

		content += MotCle.ACCESSOR_PUBLIC + MotCle.SPACE + MotCle.CLASS
				+ MotCle.SPACE + "DAO"
				+ UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom());
		content += MotCle.SPACE + MotCle.EXTEND + MotCle.SPACE + "AbstractDAO"
				+ MotCle.OPEN_ACCOLADE + MotCle.POINT_COMMA;

		content += MotCle.NEW_LINE;
		content += MotCle.INDENT;
		content += MotCle.ACCESSOR_PUBLIC + MotCle.SPACE + "DAO"
				+ UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
				+ MotCle.OPEN_PARENTHESE + MotCle.CLOSE_PARENTHESE;
		content += MotCle.OPEN_ACCOLADE + "super();"
				+ MotCle.CLOSE_ACCOLADE;

		String corps = "super.saveOrUpdate("
				+ UtilityClassFunction.firstLetterLowerCase(this.getClasse().getNom())
				+ ");";
		content += this.createMethode(MotCle.VOID, "create",
				UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
						+ MotCle.SPACE
						+ UtilityClassFunction.firstLetterLowerCase(this.getClasse()
								.getNom()), true, corps);

		corps = "super.saveOrUpdate("
				+ UtilityClassFunction.firstLetterLowerCase(this.getClasse().getNom())
				+ ");";
		content += this.createMethode(MotCle.VOID, "update",
				UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
						+ MotCle.SPACE
						+ UtilityClassFunction.firstLetterLowerCase(this.getClasse()
								.getNom()), true, corps);

		corps = "super.delete("
				+ UtilityClassFunction.firstLetterLowerCase(this.getClasse().getNom())
				+ ");";
		content += this.createMethode(MotCle.VOID, "delete",
				UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
						+ MotCle.SPACE
						+ UtilityClassFunction.firstLetterLowerCase(this.getClasse()
								.getNom()), true, corps);

		corps = "return("
				+ UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
				+ ")super.find("
				+ UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
				+ ".class,id);";
		content += this.createMethode(UtilityClassFunction.firstLetterUpperCase(this
				.getClasse().getNom()), "find", "Long id", true, corps);

		corps = "return super.findAll("
				+ UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
				+ ".class);";
		content += this.createMethode(MotCle.TYPE_LIST, "findAll", "", true,
				corps);

		corps = "return super.findAllFetch("
				+ UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
				+ ".class,listeNomAttr,modeFetch);";
		content += this.createMethode(MotCle.TYPE_LIST, "findAllFetch",
				"List listeNomAttr,String modeFetch", true, corps);

		corps = "return super.findByCriteriaEq("
				+ UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
				+ ".class,criteres);";
		content += this.createMethode(MotCle.TYPE_LIST, "findByCriteriaEq",
				"Set criteres", true, corps);

		corps = "return super.findByCriteriaEqFetch("
				+ UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
				+ ".class,criteres,listeNomAttrToFetch,fetchMode);";
		content += this.createMethode(MotCle.TYPE_LIST,
				"findByCriteriaEqFetch",
				"Set criteres,List listeNomAttrToFetch,String fetchMode", true,
				corps);

		corps = "";
		corps += "Set criteres=new HashSet();" + MotCle.NEW_LINE;
		corps += "	criteres.add(new Critere(nomAttr,valeur));"
				+ MotCle.NEW_LINE;
		corps += "	try {" + MotCle.NEW_LINE;
		corps += "		List results=this.findByCriteriaEq("
				+ UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
				+ ".class,criteres);" + MotCle.NEW_LINE;
		corps += "		return (results.size()==0) ?  null :  (results);"
				+ MotCle.NEW_LINE;
		corps += "} catch (DataAccessLayerException e) {"
				+ MotCle.NEW_LINE;
		;
		corps += "	e.printStackTrace();" + MotCle.NEW_LINE;
		corps += "}" + MotCle.NEW_LINE;
		corps += " return null;" + MotCle.NEW_LINE;
		content += this.createMethode(MotCle.TYPE_LIST, "findByAttribut",
				"String nomAttr,String valeur", true, corps);

		corps = "";
		corps += "Set criteres=new HashSet();" + MotCle.NEW_LINE;
		corps += "	criteres.add(new Critere(nomAttrCode,valeur));"
				+ MotCle.NEW_LINE;
		corps += "	try {" + MotCle.NEW_LINE;
		corps += "		List results=this.findByCriteriaEq("
				+ UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
				+ ".class,criteres);" + MotCle.NEW_LINE;
		corps += "		return (results.size()==0) ?  null :  ("
				+ UtilityClassFunction.firstLetterUpperCase(this.getClasse().getNom())
				+ ")results.get(0);" + MotCle.NEW_LINE;
		corps += "} catch (DataAccessLayerException e) {"
				+ MotCle.NEW_LINE;
		;
		corps += "	e.printStackTrace();" + MotCle.NEW_LINE;
		corps += "}" + MotCle.NEW_LINE;
		corps += " return null;" + MotCle.NEW_LINE;
		content += this.createMethode(UtilityClassFunction.firstLetterUpperCase(this
				.getClasse().getNom()), "findByCode",
				"String nomAttrCode,String valeur", true, corps);

		content += MotCle.NEW_LINE;
		content += MotCle.CLOSE_ACCOLADE;
		this.setContent(content);

	}

	/**
	 * Cette mthode cre et retourne une mthode du dao
	 * @param typeDeRetours le type de retours de la mthode
	 * @param nomMethode le nom de la mthode
	 * @param parametre les diffrents paramtres de la mthode. ex : String nom, String id
	 * @param hasThrows est ce que cette mthode doit lancer une exception
	 * @param corps le corps de la mthode
	 * @returnv cre et retourne une mthode du dao
	 */
	private String createMethode(String typeDeRetours, String nomMethode,
			String parametre, boolean hasThrows, String corps) {
		String content = "";
		content += MotCle.NEW_LINE;
		content += MotCle.NEW_LINE;
		content += MotCle.INDENT;
		content += MotCle.ACCESSOR_PUBLIC + MotCle.SPACE + typeDeRetours
				+ MotCle.SPACE + nomMethode;
		content += MotCle.OPEN_PARENTHESE + parametre;
		content += MotCle.CLOSE_PARENTHESE;
		if (hasThrows)
			content += "throws DataAccessLayerException ";
		content += MotCle.OPEN_ACCOLADE;
		content += MotCle.NEW_LINE;
		content += MotCle.INDENT;
		content += MotCle.INDENT;
		content += corps;
		content += MotCle.NEW_LINE;
		content += MotCle.INDENT;
		content += MotCle.CLOSE_ACCOLADE;

		return content;
	}

}
