/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance.classcreator;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import oued.persistance.classconcepts.Association;
import oued.persistance.classconcepts.Classe;
import oued.persistance.classconcepts.InfosModel;
import oued.persistance.classconcepts.Link;
import oued.persistance.classconcepts.SimpleProperty;
import oued.persistance.classconcepts.DataType;
import oued.persistance.mappingxml.NameResolver;
import oued.persistance.utils.UtilityClassFunction;

/**
 * Cette classe permet de crer le contenu d'une classe pojo
 * 
 * @author OUEDRAOGO Moussa
 * @since 25-01-2007
 * 
 */
public class JavaClass {
	/**
	 * La classe pour laquelle on va  crer le contenu pojo
	 */
	private Classe classe;
	
	/**
	 * Le contenu du pojo
	 */
	private String content=null;
	
	/**
	 * Le nom de la classe
	 */
	private String nomClasse;
	
	/**
	 * Le infosModele qui contient toutes les informations sur les types, associations et 
	 * autres classes
	 */
	private InfosModel infosModel;
	

	/**
	 * Le constructeur personnalis
	 * @param classe la classe pour laquelle il faut crer le contenu
	 * @param infosModel le infosModele qui contient toutes les ressources ncessaires  la cration
	 */
	public JavaClass(Classe classe,InfosModel infosModel){
		this.classe=classe;
		 nomClasse=classe.getNom()+".java";
		 this.infosModel=infosModel;
	}
	
	/**
	 * Cette mthode permet de crer le contenu du POJO
	 */
	public void createContent(){
		List<String> tempNames=new ArrayList<String>();
		String content="";
		content+=MotCle.PACKAGE+MotCle.SPACE+classe.getClassPackage()+MotCle.POINT_COMMA;
		
		content+=MotCle.NEW_LINE;
		content+=MotCle.NEW_LINE;
		
		content+=MotCle.IMPORT+MotCle.SPACE+"java.util.*"+MotCle.POINT_COMMA;
		
		content+=MotCle.NEW_LINE;
		content+=MotCle.NEW_LINE;
		
		content+=MotCle.ACCESSOR_PUBLIC+MotCle.SPACE+MotCle.CLASS+MotCle.SPACE+classe.getNom();
		//Si la classe est une classe fille. Ajouter l'instruction extends au contenu gnr
		if (classe.isChild()){
			Classe classeMere=UtilityClassFunction.motherClass(infosModel.getClasses(), classe);
			if (classeMere!=null)
				content+=MotCle.SPACE+MotCle.EXTEND+MotCle.SPACE+classeMere.getNom();
		}
		content+=MotCle.OPEN_ACCOLADE;
		
		if (!classe.isEstClasseAssociation()){//Les classes associations ne contiennent pas d'id. 
												//Verifiez que la classe ne l'est pas avant de crer l'attribut id
			content+=MotCle.NEW_LINE;
			content+=MotCle.INDENT;
			content+=MotCle.ACCESSOR_PRIVATE+MotCle.SPACE+ "long"+MotCle.SPACE+"id"+MotCle.POINT_COMMA;
			content+=this.createGetterAndSetter("id","long");
			
		}else{
			//On a une classe association. Lui ajouter un attribut qui correspond  la classe secondaire de l'association qui l'a vu naitre
			Classe classeSecondaire=UtilityClassFunction.secondaryClassForLinkedClass(infosModel.getAssociations(),infosModel.getClasses(), classe);
			content+=MotCle.NEW_LINE;
			content+=MotCle.INDENT;
			content+=MotCle.ACCESSOR_PRIVATE+MotCle.SPACE+UtilityClassFunction.firstLetterUpperCase(classeSecondaire.getNom())+MotCle.SPACE+UtilityClassFunction.firstLetterLowerCase(classeSecondaire.getNom())+MotCle.POINT_COMMA;
			content+=this.createGetterAndSetter(UtilityClassFunction.firstLetterLowerCase(classeSecondaire.getNom()),UtilityClassFunction.firstLetterUpperCase(classeSecondaire.getNom()));
		}
//		cration du contenu des proprits simples
		Iterator< SimpleProperty>itAttr=classe.getProprietesSimples().iterator();
		while (itAttr.hasNext()){
			SimpleProperty proprieteSimple=(SimpleProperty)itAttr.next();
			content+=MotCle.NEW_LINE;
			content+=MotCle.INDENT;
			String type=null;
			//Convertir le type de l'attribut au bon type avant de gnrer
			if (proprieteSimple.getIdType()!=null)				
				type=UtilityClassFunction.convertirWrappeType((String)infosModel.getTypes().get(proprieteSimple.getIdType()));
			else
				type=UtilityClassFunction.convertirWrappeType(DataType.STRING);
			
			content+=MotCle.ACCESSOR_PRIVATE+MotCle.SPACE+type+MotCle.SPACE+proprieteSimple.getName()+MotCle.POINT_COMMA;	
		
			content+=this.createGetterAndSetter(proprieteSimple.getName(),type);
			content+=MotCle.NEW_LINE;
		}
		//Generation des association;
		NameResolver nameResolver=new NameResolver(tempNames);
		Iterator<Association>itAssociation=classe.getAssociations().iterator();
		while(itAssociation.hasNext()){
			Association association=(Association) itAssociation.next();
			//Cas des associations simples
			if ( (association.getLinkType().compareTo(Association.SIMPLE_ASSOCIATION)==0)
					|| (association.getLinkType().compareTo(Association.AGREGATION)==0)
					|| (association.getLinkType().compareTo(Association.COMPOSITION)==0)){
				
				content+=this.createContentSimpleAssociation(association, tempNames, nameResolver);
				tempNames.add(nameResolver.getName());
			}

			/**
			 * Cas des classes associations
			 * 
			 */
			if ( (association.getLinkType().compareTo(Association.CLASSE_ASSOCIATION)==0)){
				//Si cette classe est la classe principale, y ajouter l'attribut
				Classe linkedClasse=association.getClassLinked();
				Classe classePrincipale=UtilityClassFunction.principalClassForClassAssociation(infosModel.getClasses(), classe, association);
				//Si c'est vrai. C'est qu'on ai dans la classe principale
				if (classe.equals(classePrincipale)){
					content+=MotCle.NEW_LINE;
					content+=MotCle.INDENT;
					content+=MotCle.ACCESSOR_PRIVATE+MotCle.SPACE+MotCle.TYPE_LIST+MotCle.SPACE+"liste"+UtilityClassFunction.firstLetterUpperCase(linkedClasse.getNom())+MotCle.POINT_COMMA;
					content+=this.createGetterAndSetter("liste"+UtilityClassFunction.firstLetterUpperCase(linkedClasse.getNom()),"List");
					
				}else{
					content+=MotCle.NEW_LINE;
					content+=MotCle.NEW_LINE;
					content+=MotCle.INDENT;
					content+=MotCle.ACCESSOR_PRIVATE+MotCle.SPACE+UtilityClassFunction.firstLetterUpperCase(linkedClasse.getNom())+MotCle.SPACE+UtilityClassFunction.firstLetterLowerCase(linkedClasse.getNom())+MotCle.POINT_COMMA;
					content+=this.createGetterAndSetter(UtilityClassFunction.firstLetterLowerCase(linkedClasse.getNom()),UtilityClassFunction.firstLetterUpperCase(linkedClasse.getNom()));
				}
					
			}
			
		}
		
		content+=MotCle.NEW_LINE;
		content+=MotCle.CLOSE_ACCOLADE;
		this.content=content;
	}
	
	/**
	 * Cette mthode retourne la  classe pour laquelle on gnre le contenu
	 * @return la  classe pour laquelle on gnre le contenu
	 */
	public Classe getClasse() {
		return classe;
	}

	/**
	 * Cette mthode modifie la  classe pour laquelle on gnre le contenu
	 * @param classe la  classe pour laquelle on gnre le contenu
	 */
	public void setClasse(Classe classe) {
		this.classe = classe;
	}

	/**
	 * Cette mthode retourne le contenu de la  classe 
	 * @return le contenu de la  classe 
	 */
	public String getContent() {
		return content;
	}

	/**
	 * Cette mthode modifie le contenu de la classe
	 * @param content le nouveau contenu de la classe
	 */
	public void setContent(String content) {
		this.content = content;
	}
	

	

	/**
	 * Cette mthode retourne le nom de la classe POJO  gnrer
	 * @return le nom de la classe POJO  gnrer
	 */
	public String getNomClasse() {
		return nomClasse;
	}

	/**
	 * Cette mthode modifie le nom de la classe pojo
	 * @param nomClasse le nouveau nom de la classe pojo
	 */
	public void setNomClasse(String nomClasse) {
		this.nomClasse = nomClasse;
	}
	
	/**
	 * Cette mthode cr le contenu correspondant au getter et setter de l'attribut qui lui est pass en paramtre. I
	 * @param nomAttribut le nom de l'attribut pour lequel il faut crer le getter et le setter
	 * @param type le type de l'attribut
	 * @return le contenu correspondant au getter et setter de l'attribut qui lui est pass en paramtre. I
	 */
	private String createGetterAndSetter(String nomAttribut,String type){
		//Creer le contenu du getter
		String contenuTemp="";
		contenuTemp+=MotCle.NEW_LINE;
		contenuTemp+=MotCle.INDENT;
		contenuTemp+=MotCle.ACCESSOR_PUBLIC+MotCle.SPACE+type+MotCle.SPACE+MotCle.GETTER+UtilityClassFunction.firstLetterUpperCase(nomAttribut);
		contenuTemp+=MotCle.OPEN_PARENTHESE+MotCle.CLOSE_PARENTHESE+MotCle.OPEN_ACCOLADE;
		contenuTemp+=MotCle.RETURN+MotCle.SPACE+nomAttribut+MotCle.POINT_COMMA;
		contenuTemp+=MotCle.CLOSE_ACCOLADE;
		
		
		//Creer le contenu du setter
		contenuTemp+=MotCle.NEW_LINE;
		contenuTemp+=MotCle.INDENT;
		contenuTemp+=MotCle.ACCESSOR_PUBLIC+MotCle.SPACE+MotCle.VOID+MotCle.SPACE+MotCle.SETTER+UtilityClassFunction.firstLetterUpperCase(nomAttribut);
		contenuTemp+=MotCle.OPEN_PARENTHESE+type+MotCle.SPACE+nomAttribut+MotCle.CLOSE_PARENTHESE+MotCle.OPEN_ACCOLADE;
		contenuTemp+=MotCle.THIS+MotCle.POINT+nomAttribut+MotCle.EQUALITY+nomAttribut+MotCle.POINT_COMMA;
		contenuTemp+=MotCle.CLOSE_ACCOLADE;
		contenuTemp+=MotCle.NEW_LINE;
		
		return contenuTemp;
	}
	
	private  String createContentSimpleAssociation(Association association,List<String> tempNames,NameResolver nameResolver){
		//Recuperer l'autre bout de l'association
		String content="";
		Classe otherClasse=UtilityClassFunction.otherAssociationClass(infosModel.getClasses(),classe, association);
		Link otherClassLink= UtilityClassFunction.classLink(otherClasse, association);
		Link currentClassLink= UtilityClassFunction.otherClassLink(otherClasse, association);
		String nom="";
		nom=nameResolver.retrieveName(otherClassLink, currentClassLink, otherClasse);
		if (otherClassLink.getMultiplicity().isOne_One()){//				
			content+=MotCle.NEW_LINE;
			content+=MotCle.INDENT;
			content+=MotCle.ACCESSOR_PRIVATE+MotCle.SPACE+UtilityClassFunction.firstLetterUpperCase(otherClasse.getNom())+MotCle.SPACE+UtilityClassFunction.firstLetterLowerCase(nom)+MotCle.POINT_COMMA;
			content+=this.createGetterAndSetter(UtilityClassFunction.firstLetterLowerCase(nom),UtilityClassFunction.firstLetterUpperCase(otherClasse.getNom()));
			
			
		}else if (otherClassLink.getMultiplicity().isManyToMany()){
			if (currentClassLink.getMultiplicity().isOne_One()){
				content+=MotCle.NEW_LINE;
				content+=MotCle.INDENT;
				content+=MotCle.ACCESSOR_PRIVATE+MotCle.SPACE+MotCle.TYPE_SET+MotCle.SPACE+nom+MotCle.POINT_COMMA;
				content+=this.createGetterAndSetter(nom,"Set");
				
			}else if (currentClassLink.getMultiplicity().isManyToMany()){//					
				content+=MotCle.NEW_LINE;
				content+=MotCle.INDENT;
				content+=MotCle.ACCESSOR_PRIVATE+MotCle.SPACE+MotCle.TYPE_SET+MotCle.SPACE+nom+MotCle.POINT_COMMA;
				content+=this.createGetterAndSetter(nom,"Set");
			}
		}
		return content;
	}
	
	
	
	
	
	
}
