/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package oued.persistance.classcreator;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import oued.persistance.utils.UtilityClassFunction;

/**
 * Cette classe permet la cration de fichiers java  sur le disque dur
* @author OUEDRAOGO Moussa
 * @since 28-01-2007
 *
 */
public class PhysicalClassCreator {
	/**
	 * Le package des diffrentes classes  crer
	 */
//	private String packageClasse;
	/**
	 * Le repertoire de travail
	 */
	private String workingDir;
	/**
	 * Le repertoire des sources. ce repertoire est relatif au repertoire de travail
	 */
	private String srcDir;
	
	
	/**
	 * Constructeur personnalis du {@link PhysicalClassCreator}
	 * @param workingDir le repertoire de travail
	 * @param srcDir le repertoire des sources, relatif au repertoire de travail
	 */
	public PhysicalClassCreator(String workingDir,String srcDir){
		this.workingDir=workingDir;
		this.srcDir=srcDir;
	}
	
	/**
	 * Cette mthode permet de crer physiquement les fichiers sur le disque
	 * @throws CreateClasseException
	 */
	public void createClasse(JavaClass javaContent) throws IOException{
		if (javaContent==null||javaContent.getContent()==null)
			throw new IllegalArgumentException("L'objet javaContent ne peut tre null. Son contenu ne peut tre aussi null");
		String nomComplet="";
		//Le nom complet du fichier (chemin + nom du fichier)
		if (this.srcDir!=null && this.srcDir.trim().compareTo("")!=0)
			nomComplet=this.workingDir+"\\"+this.srcDir+"\\"+UtilityClassFunction.replacePointBySlash(javaContent.getClasse().getClassPackage())+"\\"+javaContent.getNomClasse();
		else
			nomComplet=this.workingDir+"\\"+UtilityClassFunction.replacePointBySlash(javaContent.getClasse().getClassPackage())+"\\"+javaContent.getNomClasse();
		
		/**
		 * Creer le fichier correspondant
		 * 
		 */
		File f=new File(nomComplet);		
		f.createNewFile();//Creer un fichier vide sur les disque		
		FileOutputStream fos=null;		
		fos = new FileOutputStream(f);			
		BufferedOutputStream bos = new BufferedOutputStream(fos);
		PrintWriter dos = new PrintWriter(bos);	
		//Ecrire dans le fichier vide le contenu de JavaContent
		dos.write(javaContent.getContent());			
		dos.flush();
		dos.close();		
	}


	
	
}
