/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package oued.persistance.configuration;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;


/**
 * Cette Classe permet d'encapser tous les paramtres personnaliss concernant
 * un contexte 
 * @author ouedmouss
 * @author OUEDRAOGO Moussa
 * @since 05-05-2007
 *
 */
public class Configurator {
	/**
	 * Le nom de l'attribut ventuel  qui appartient ce configuration
	 */
	private String nomAttribut;
	/**
	 * Les paramtres du {@link Configurator}
	 */
	private String parametres;
	
	/**
	 * Les diffrents lments contenu dans paramtres
	 */
	private List<String> eltsParametres;
	/**
	 * Les diffrents valeurs des lments contenu dans paramtres
	 */
	private List<String> valeursParametres;
	
	/**
	 * Reprsente le contexte du Configuration
	 */
	private Context mappingContext;
	
	/**
	 * Va servir  hberger les diffrents attributs de nomAttribut dans lequel ou ce dernier
	 * en contiendrait plusieurs. C'est le cas pour les contexts COMPONENT, PROPERTIES, NATURAL_ID
	 */
	private List<String> listeNomAttr=null;
	
	/**
	 * Constructeur par dfaut
	 */
	public Configurator(){
		eltsParametres=new ArrayList<String>();
		valeursParametres=new ArrayList<String>();
	}
	/**
	 * retourne le nom de l'attribut proprit ventuel du {@link Configurator}
	 * @return le nom de l'attribut proprit ventuel du {@link Configurator}
	 */
	public String getNomAttribut() {
		return nomAttribut;
	}
	
	/**
	 * modifie le nom de l'attribut propritaire ventuel du {@link Configurator}
	 * @param nomAttribut le nom de l'attribut propritaire ventuel du {@link Configurator}
	 */
	public void setNomAttribut(String nomAttribut) {
		this.nomAttribut = nomAttribut;
	}
	
	/**
	 * retourne le context du Configuration
	 * @return le context du Configuration
	 */
	public Context getMappingContext() {
		return mappingContext;
	}
	
	/**
	 * modifie le context du Configuration
	 * @param  mappingContext le nouveau context du Configuration
	 */
	public void setMappingContext(Context mappingContext) {
		this.mappingContext = mappingContext;
	}
	
	/**
	 * Recupre les proprits qui se trouve dans parametre et positionne les liste
	 * eltsParametres et valeursParametres
	 * @throws ConfigurationBuildException Exception lance lorsqu'une erreur survient
	 */
	public void buildProprietes() throws ConfigurationBuildException{
		StringTokenizer tokenizer_config=new StringTokenizer(parametres,",");
		String tokenCourant="";
		try{
			while (tokenizer_config.hasMoreTokens()){
				tokenCourant=tokenizer_config.nextToken();
				StringTokenizer tokenizer_elt=new StringTokenizer(tokenCourant,"=");
				
				String propriete=tokenizer_elt.nextToken();
				String valeur=tokenizer_elt.nextToken();
				valeur=valeur.substring(1,valeur.length()-1);
				
				eltsParametres.add(propriete);
				valeursParametres.add(valeur);
			}
		}catch(Exception e){
			throw new ConfigurationBuildException("Erreur prs de "+tokenCourant+" pour le context "+this.getMappingContext().getContext());
		}
	}
	
	/**
	 * retourne vrai si le nom de l'attribut pass en paramtre se trouve dans nomAttribut et faux sinon
	 * @param nomAttr  l'attribut  vrifier l'existence
	 * @return vrai si le nom de l'attribut pass en paramtre se trouve dans nomAttribut et faux sinon
	 */
	public boolean contientAttribut(String nomAttr){
		if (listeNomAttr==null)
			this.positiontionnerElementsAttribut();
		
		return listeNomAttr.contains(nomAttr);
	}
	
	/**
	 * retourne le parametre du {@link Configurator}
	 */
	public String getParametres() {
		return parametres;
	}
	/**
	 * modifie le parametre du {@link Configurator}
	 */
	public void setParametres(String parametres) {
		this.parametres = parametres;
	}
	
	/**
	 * retourne le parametre du {@link Configurator}
	 * @return le parametre du {@link Configurator}
	 */
	public List<String> getEltsParametres() {
		return eltsParametres;
	}
	
	
	/**
	 * retourne les diffrentes valeurs des lments contenu dans paramtres
	 * @return les diffrents valeurs des lments contenu dans paramtres
	 */
	public List<String> getValeursParametres() {
		return valeursParametres;
	}
	
	/**
	 * positionner la variable listeNomAttr. Pour rappel, cette variable permet de conserver l'ensemble
	 * des noms d'attributs dans le cas ou l'on est dans les contextes PROPERTIES, COMPONENT, PROPERTIES.
	 */
	private void positiontionnerElementsAttribut(){
		listeNomAttr=new ArrayList<String>();
		StringTokenizer tokenizer=new StringTokenizer(nomAttribut,",");
		while(tokenizer.hasMoreElements()){
			listeNomAttr.add(tokenizer.nextToken());
		}
	}
	
}
