/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package oued.persistance.configuration;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

/**
 * Cette classe reprsente la configuration d'un projet PersistanceZero. C'est cette classe
 * qui encapsule l'ensemble des paramtres utilisateurs pour toutes les classes et tous les 
 * contexte
 * @author OUEDRAOGO Moussa
 * @since 05-06-2007
 *
 */
public class PersistanceZeroConfigurator {
	/**
	 * Constante qui reprsente les diffrents contextes
	 */
	public static String HIBERNATE_MAPPING_CONTEXT="HIBERNATE_MAPPING_CONTEXT";
	public static String CLASS="CLASS_CONTEXT";
	public static String VERSION="VERSION_CONTEXT";
	public static String ID="ID_CONTEXT";
	public static String PROPERTY="PROPERTY_CONTEXT";
	public static String MANY_TO_ONE="MANY_TO_ONE_CONTEXT";
	public static String PROPERTIES="PROPERTIES_CONTEXT";
	public static String NATURAL_ID="NATURAL_ID_CONTEXT";
	public static String COMPONENT_CONTEXT="COMPONENT_CONTEXT";
	public static String COLLECTION="COLLECTION_CONTEXT";
	public static String KEY_ELEMENT="KEY ELEMENT";
	
	/**
	 * L'element configuration global au proejt
	 */
	private List<Configurator>  hibernateCfgConfigurators;
	/**
	 * L'ensemble des Configurations de toutes les classes
	 */
	private Hashtable<String,ClassConfigurator> classConfigurators;
	
	/**
	 * Constructeur par dfaut
	 */
	public PersistanceZeroConfigurator(){
		classConfigurators=new Hashtable<String, ClassConfigurator>();
		hibernateCfgConfigurators=new ArrayList<Configurator>();
		
	}
	/**
	 * Permet de dfinir les paramtres globaux  PersistanceZeri
	 * @param context Le context des paramtres
	 * @param parametres l'ensemble des paramtres globaux
	 */
	public void defineHibernateCfgParameter(String context,String parametres){
		Configurator configuration=new Configurator();
		configuration.setParametres(context+"='"+parametres+"'");		
		this.hibernateCfgConfigurators.add(configuration);
	}
	
	/**
	 * ajoute un {@link Configurator}  au projet
	 * @param cc le {@link ClassConfigurator}  ajouter
	 */
	public void addClassConfigurator(ClassConfigurator cc){
		if (cc==null) throw new NullPointerException("Le classeConfigurateur ne peut pas tre null");
		if (cc.getNomClasse()==null) throw new NullPointerException("Le nom de la classe li  classeConfigurateur ne peut pas tre null");
		classConfigurators.put(cc.getNomClasse(),cc);
	}
	
	/**
	 * retourne le {@link Configurator} global au projet de persistance
	 * @return le {@link Configurator} global au projet de persistance
	 */

	
	public Hashtable<String, ClassConfigurator> getConfigurators() {
		return classConfigurators;
	}
	
	/**
	 * modifie l'ensemble des {@link Configurator} de toutes les classes
	 * @param configurators l'ensemble des {@link Configurator} de toutes les classes
	 */
	public void setConfigurators(Hashtable<String, ClassConfigurator> configurators) {
		this.classConfigurators = configurators;
	}
	
	/**
	 * retourne le {@link ClassConfigurator} de la classe passe en paramtre
	 * @param nomClasse le nom de la classe pour laquelle on cherche le {@link ClassConfigurator}
	 * @return le {@link ClassConfigurator} de la classe passe en paramtre
	 */
	public ClassConfigurator getClasseConfigurator(String nomClasse){
		return this.classConfigurators.get(nomClasse);
	}
	
	/**
	 * retourne l'ensemble des {@link Configurator} de toutes les classes
	 * @return l'ensemble des {@link Configurator} de toutes les classes
	 */
	public List<Configurator> getGlobalsConfigurators() {
		return hibernateCfgConfigurators;
	}
	
	/**
	 * modifie la liste des Configurator globaux au projet de persistance
	 * @param globalsConfigurators la nouvelle liste des Configurators globaux du projet
	 */
	public void setGlobalsConfigurators(List<Configurator> globalsConfigurators) {
		this.hibernateCfgConfigurators = globalsConfigurators;
	}
	
	
	
}
