/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance.infosmodelgenerator;

import java.util.ArrayList;
import java.util.List;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Cette classe reprsente un proxy pour un Objet Document.
 * Il fournit les informations sur le fichiers xmi
 * @author OUEDRAOGO Moussa
 * @since 14-03-2007
 *
 */
public class DocumentProxy2_1 implements IDocumentProxy {
	/**
	 * le document encapsul
	 */
	private Document document;
	
	/**
	 * Constante des maux text. Lorsqu'un nom de node est qual  ce test.
	 * C'est qu'il n'est pas correcte
	 */
	private static String badText="#text";
	
	
	/**
	 * Le constructeur personnalis
	 * @param document le document 
	 */
	public DocumentProxy2_1(Document document){
		this.document=document;
	}
	
	/**
	 * retourne le premier Node de l'objet node pass en paramtre
	 * @param node le node
	 * @return le premier Node de l'objet node pass en paramtre
	 */
	public Node getFirstChild(Node node){		
		if (node==null) return null; 
		Node temp=node.getFirstChild();
		if (temp==null) return null;
		String name=temp.getNodeName();
		
		while(name.compareTo(badText)==0){
			temp=temp.getNextSibling();
			if (temp==null) return null;
			name=temp.getNodeName();
		}
		return temp;
	}
	
	/**
	 * retourne le node suivant du node pass en paramtre
	 * @param node le node
	 * @return le node suivant du node pass en paramtre
	 */
	public Node getNextSibling(Node node){		
		if (node==null) return null; 
		Node temp=node.getNextSibling();
		if (temp==null) return null;
		String name=temp.getNodeName();
		
		while(name.compareTo(badText)==0){
			temp=temp.getNextSibling();
			if (temp==null) return null;
			name=temp.getNodeName();
		}
		return temp;
	}
	
	/**
	 * retourne l'ensemble des dats qui correspondent   une association
	 * @return l'ensemble des dats qui correspondent   une association
	 */
	public List<Node> getAssociationsTags(){
		List<Node> results_1;
		List<Node> results_2;
		List<Node> results_3;
		List<Node> results_4;
		results_1=this.getTags("ownedElement", "xmi:type", "uml:Association");
		results_2=this.getTags("packagedElement", "xmi:type", "uml:Association");
		results_3=this.getTags("ownedElement", "xmi:type", "uml:AssociationClass");
		results_4=this.getTags("packagedElement", "xmi:type", "uml:AssociationClass");
		
		results_1.addAll(results_2);
		results_1.addAll(results_3);
		results_1.addAll(results_4);
		return results_1;
		
	}
	
	
	/**
	 * retourne l'ensemble des dats qui correspondent   un type
	 * @return l'ensemble des dats qui correspondent   un type
	 */
	public List<Node> getTypesTags(){
		List<Node> results_1;
		List<Node> results_2;
		results_1=this.getTags("ownedMember", "xmi:type", "uml:DataType");
		results_2=this.getTags("packagedElement", "xmi:type", "uml:DataType");
		results_1.addAll(results_2);
		return results_1;
	}
	
	/**
	 * retourne l'ensemble des dats qui correspondent   une classe
	 * @return l'ensemble des dats qui correspondent   une classe
	 */
	public List<Node> getClassesTags(){
		List<Node> results_1;
		List<Node> results_2;
		results_1=this.getTags("ownedMember", "xmi:type", "uml:Class");
		results_2=this.getTags("packagedElement", "xmi:type", "uml:Class");
		results_1.addAll(results_2);
		return results_1;
	}
	
	
	/**
	 * retourne le Document encapsul
	 * @return le Document encapsul
	 */
	public Document getDocument() {
		return document;
	}

	/**
	 * modifie le Document encapsul
	 * @param document le nouveau  Document  encapsuler
	 */
	public void setDocument(Document document) {
		this.document = document;
	}
	
	
	/**
	 * retourne la valeur de l'attribut attribut du Node node
	 * @param node le node indiqu
	 * @param attribut le nom de l'attribut pour lequel on doit connaitre la valeur
	 * @return la valeur de l'attribut attribut du Node node
	 */
	public String getAttributValue(Node node,String attribut){
		Node nodeAttrType=node.getAttributes().getNamedItem(attribut);
		String nom=null;
		if (nodeAttrType!=null)
			nom=nodeAttrType.getNodeValue();
		return nom;
		
	}
	
	/**
	 * retourne un Element par son id
	 * @param id le id de l'lment
	 * @return un Element par son id
	 */
	public Element getElementById(String id){
		return document.getElementById(id);
	}
	
	/**
	 * retourne l'ensemble des Node selon les critres passs en paramtre
	 * @param nomTag le nom du tag
	 * @param typeAttribut le type d'attribut
	 * @param valeurAttribut la valeur de l'attribut
	 * @return
	 */
	private List<Node> getTags(String nomTag,String typeAttribut,String valeurAttribut){
		NodeList tempps=document.getElementsByTagName(nomTag);
		List<Node> results=new ArrayList<Node>();
		for (int i=0;i<tempps.getLength();i++){
			Node elt=tempps.item(i);
			Node nodeAttrType=null;
			if (elt.hasAttributes())
				nodeAttrType=elt.getAttributes().getNamedItem(typeAttribut);
				if (nodeAttrType!=null
						&& nodeAttrType.getNodeValue().compareTo(valeurAttribut)==0){
					results.add(elt);	
				}
			}
		return results;
	}
}