/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance.infosmodelgenerator;

import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Interface pour encapsuler le proxy sur l'objet Document
 * @author ouedmouss
 * @author OUEDRAOGO Moussa
 * @since 14-03-2007
 *
 */
public interface IDocumentProxy {
	/**
	 * retourne le premier Node de l'objet node pass en paramtre
	 * @param node le node
	 * @return le premier Node de l'objet node pass en paramtre
	 */
	public Node getFirstChild(Node node);
	/**
	 * retourne le node suivant du node pass en paramtre
	 * @param node le node
	 * @return le node suivant du node pass en paramtre
	 */
	public Node getNextSibling(Node node);
	
	/**
	 * retourne l'ensemble des tags qui correspondent   une association
	 * @return l'ensemble des dats qui correspondent   une association
	 */
	public List<Node> getAssociationsTags();
	
	/**
	 * retourne l'ensemble des dats qui correspondent   un type
	 * @return l'ensemble des dats qui correspondent   un type
	 */
	public List<Node> getTypesTags();
	
	/**
	 * retourne l'ensemble des dats qui correspondent   une classe
	 * @return l'ensemble des dats qui correspondent   une classe
	 */
	public List<Node> getClassesTags();
	
	/**
	 * modifie le Document encapsul
	 * @param document le nouveau  Document  encapsuler
	 */
	public void setDocument(Document document);
	
	/**
	 * retourne la valeur de l'attribut attribut du Node node
	 * @param node le node indiqu
	 * @param attribut le nom de l'attribut pour lequel on doit connaitre la valeur
	 * @return la valeur de l'attribut attribut du Node node
	 */
	public String getAttributValue(Node node,String attribut);
	
	/**
	 * retourne un Element par son id
	 * @param id le id de l'lment
	 * @return un Element par son id
	 */
	public Element getElementById(String id);
	
}
