/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package oued.persistance.infosmodelgenerator;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;


import oued.persistance.classconcepts.Association;
import oued.persistance.classconcepts.Classe;
import oued.persistance.classconcepts.InfosModel;
import oued.persistance.classconcepts.Link;
import oued.persistance.classconcepts.Multiplicity;
import oued.persistance.classconcepts.SimpleProperty;
import oued.persistance.infosmodelgenerator.transformer.ClassAssociationTransformer;
import oued.persistance.infosmodelgenerator.transformer.ITransformer;
import oued.persistance.infosmodelgenerator.transformer.InheritanceTransformer;

/**
 * Classe permettant la rcupration des diffrentes informations du fichier XMI 
 * de version 2.1
 * @author OUEDRAOGO Moussa
 * @since 02-04-2007
 *
 */
public class InfosGeneratorXMI2_1  implements IInfosGenerator{

	private Hashtable<String,Classe> classes;
	private Hashtable<String,Association>  associations;
	private Hashtable<String,String> types;

	/**
	 * Le constructeur personnalis
	 */
	public InfosGeneratorXMI2_1(){
		this.setClasses(new Hashtable<String,Classe>());
		this.setAssociations(new Hashtable<String,Association>());
		this.setTypes(new Hashtable<String,String>());
	}
	/**
	 * Lancer la cration et la rcupration des donnes
	 * @param  nomFichier le fichier xmi
	 */
	public void produceInfosModel(String classPackage,String nomFichier) throws XMIException{
		DocumentBuilderFactory fabrique = DocumentBuilderFactory.newInstance();
		DocumentBuilder constructor=null;
		Document document=null;
		DocumentProxy2_1 docProxy;
		
			try {
				constructor = fabrique.newDocumentBuilder();
				File f=new File(nomFichier);
				document = constructor.parse(f);	
				docProxy=new DocumentProxy2_1(document);
				
				
				
				//Enregistrer les associations
				List<Node> associations=docProxy.getAssociationsTags();
				Iterator<Node> it=associations.iterator();
				while (it.hasNext()){
					Node node=(Node)it.next();
					Association association=this.transformTagToAssociation(node, docProxy);
					this.getAssociations().put(association.getId(), association);
				}		
				
				//Creation des types de donnes
				List<Node> types=docProxy.getTypesTags();
				Iterator<Node> it_1=types.iterator();
				while (it_1.hasNext()){
					Node node=(Node)it_1.next();
					this.addType(node, docProxy);
				}
				
				//Creation des classes
				List<Node> classes=docProxy.getClassesTags();
				Iterator<Node> it_2=classes.iterator();
				while (it_2.hasNext()){
					Node node=(Node)it_2.next();
					Classe classe=this.transformTagToClass(node, docProxy);
					Set<SimpleProperty> attributs=this.getAttributsClasse(node, docProxy);
					Set<Association> assx=this.getAssociationsDeClasse(node, docProxy);
					classe.setAssociations(assx);
					classe.setProprietesSimples(attributs);
					Association generalisation=this.getGeneralization(node, docProxy);
					if (generalisation!=null){
						Link child=new Link();
						child.setClassId(classe.getId());
						generalisation.setEnd(child);
						classe.getAssociations().add(generalisation);
						this.getAssociations().put(generalisation.getId(),generalisation);
						classe.setChild(true);
					}
					classe.setClassPackage(classPackage);
					this.getClasses().put(classe.getId(),classe);
				}
				
				this.positionnerAssociationsDesClasses(classPackage);
				
			} 
			catch (ParserConfigurationException e) {
				e.printStackTrace();
				e.printStackTrace();
			} catch (SAXException e) {
				e.printStackTrace();
				new XMIException("Impossible de parser le fichier xmi."+e.getMessage());
			} catch (IOException e) {
				e.printStackTrace();
				new XMIException("Impossible de parser le fichier xmi."+e.getMessage());
			}catch(Exception e){
				new XMIException("Impossible de parser le fichier xmi."+e.getMessage());
			}
			
	}
	
	
	/**
	 * Transforme le Node node pass en paramtre en Objet Association et retourne cet objet
	 * @param node le node concern
	 * @param docProxy le proxy sur l'objet document
	 * @return un Association correspondant au Node node pass en paramtre
	 */
	private Association transformTagToAssociation(Node node,DocumentProxy2_1 docProxy){
		String type=docProxy.getAttributValue(node,"xmi:type");
		String id=docProxy.getAttributValue(node,"xmi:id");
		String nomAss=docProxy.getAttributValue(node,"name");
		Association association=new Association();
			association.setLinkType(Association.SIMPLE_ASSOCIATION);
		if (type.compareTo("uml:AssociationClass")==0)
			association.setLinkType(Association.CLASSE_ASSOCIATION);
		
		association.setId(id);		
		association.setName(nomAss==null ? "" : nomAss);
		Node memberEnd_1=docProxy.getFirstChild(node);
		Node memberEnd_2=docProxy.getNextSibling(memberEnd_1);
		
		String idref1=docProxy.getAttributValue(memberEnd_1, "xmi:idref");
		String idref2=docProxy.getAttributValue(memberEnd_2, "xmi:idref");
		
		if (association.getLinkType().compareTo(Association.CLASSE_ASSOCIATION)==0){			
		Classe linkedClasse=new Classe();
		linkedClasse.setId(idref2);
		association.setClassLinked(linkedClasse);		
		association.setIdAssocRef(idref1);
		}
		return association;
			
	}
	
	/**
	 * ajoute un nouveau type  la liste des types
	 * @param node le node ou l'on recupere le type
	 * @param docProxy le proxy sur l'objet document
	 */
	private void addType(Node element,DocumentProxy2_1 docProxy){
		String type=docProxy.getAttributValue(element,"name" );
		String id=docProxy.getAttributValue(element,"xmi:id" );
		this.getTypes().put(id,type);
	}
	
	/**
	 * Transforme le Node node pass en paramtre en Objet Classe et retourne cet objet
	 * @param node le node concern
	 * @param docProxy le proxy sur l'objet document
	 * @return une Classe correspondant au Node node pass en paramtre
	 */
	private Classe transformTagToClass(Node node,DocumentProxy2_1 docProxy){		
		String name=docProxy.getAttributValue(node,"name");
		String id=docProxy.getAttributValue(node,"xmi:id");
		String visibilite=docProxy.getAttributValue(node,"visibility");
		Classe classe=new Classe();
		classe.setId(id);
		classe.setVisibilite(visibilite);
		classe.setNom(name);
		return classe;
	
	}
	
	/**
	 * retourne tous les attributs du Node node. ce node correspond au tag d'une classe
	 * @param node le node concern
	 * @param docProxy le proxy sur l'objet document
	 * @return tous les attributs du Node node. ce node correspond au tag d'une classe
	 */
	private Set<SimpleProperty> getAttributsClasse(Node element,DocumentProxy2_1 docProxy){
		Node child=docProxy.getFirstChild(element);
		boolean hasNext=true;
		if (child==null) hasNext=false;
		Set<SimpleProperty> results=new HashSet<SimpleProperty>();
		while(hasNext){
			String type=docProxy.getAttributValue(child,"xmi:type");
			String associationType=docProxy.getAttributValue(child,"association");
			if (type.compareTo("uml:Property")!=0){
				child=docProxy.getNextSibling(child);
				if (child==null) hasNext=false;
				continue;
			}
			if (associationType!=null) {
				child=docProxy.getNextSibling(child);
				if (child==null) hasNext=false;
				continue;
			}
			
			String name=docProxy.getAttributValue(child,"name");
			String id=docProxy.getAttributValue(child,"xmi:id");
			String visibilite=docProxy.getAttributValue(child,"visibility");
			
			SimpleProperty attribut=new SimpleProperty();
			attribut.setId(id);
			attribut.setName(name);
			attribut.setVisibility(visibilite);
			
			//Recuperer le type de donnes
			String typeDonnees=null;
			if (typeDonnees!=null){
				Node nodeTypeDonnees=docProxy.getFirstChild(child);
				typeDonnees=docProxy.getAttributValue(nodeTypeDonnees, "xmi:idref");			
				attribut.setIdType(typeDonnees);
			}else
				attribut.setIdType(null);
			results.add(attribut);
			
			child=docProxy.getNextSibling(child);
			if (child==null) hasNext=false;

		}
		return results;
	}
	
	/**
	 * retourne tous les associations du Node node. ce node correspond au tag d'une classe
	 * @param node le node concern
	 * @param docProxy le proxy sur l'objet document
	 * @return toutes  les associations du Node node. ce node correspond au tag d'une classe
	 */
	private Set<Association> getAssociationsDeClasse(Node element,DocumentProxy2_1 docProxy){
		
		Node child=docProxy.getFirstChild(element);
		boolean hasNext=true;
		if (child==null) hasNext=false;
		Set<Association> associations=new HashSet<Association>();
		while(hasNext){
			String type=docProxy.getAttributValue(child,"xmi:type");
			String associationType=docProxy.getAttributValue(child,"association");
			if (type.compareTo("uml:Property")!=0){
				child=docProxy.getNextSibling(child);
				if (child==null) hasNext=false;
				continue;
			}
			if (associationType==null) {
				child=docProxy.getNextSibling(child);
				if (child==null) hasNext=false;
				continue;
			}
			
			String roleNoun=docProxy.getAttributValue(child,"name");
			//String id=docProxy.getAttributValeur(child,"xmi:id");
			
			
			//Recuperer la classe referencee par cette association
			Node typeClasseRefTag=docProxy.getFirstChild(child);
			String idClasseRef=docProxy.getAttributValue(typeClasseRefTag,"xmi:idref");
			
			String cardMin=null,cardMax=null;
			//Recuperer les cardinalites
			Node cardTag1=docProxy.getNextSibling(typeClasseRefTag);
			Node cardTag2=docProxy.getNextSibling(cardTag1);
			String nomTag1=null;
			if (cardTag1!=null){
				nomTag1=cardTag1.getNodeName();
				if (nomTag1.compareTo("lowerValue")==0){
					cardMin=docProxy.getAttributValue(cardTag1, "value");
					cardMax=docProxy.getAttributValue(cardTag2, "value");
				}else{
					cardMin=docProxy.getAttributValue(cardTag2, "value");
					cardMax=docProxy.getAttributValue(cardTag1, "value");
				}
			}	
			
			//Recuperer l'association par son id
			//NB: le id de l'association est heberge dans le tag association
			Association association=(Association)this.getAssociations().get(associationType);
			if (association==null) continue;
			Link lien=new Link();
			Multiplicity multiplicity=new Multiplicity();
			multiplicity.setCardMin(cardMin);
			multiplicity.setCardMax(cardMax);
			lien.setMultiplicity(multiplicity);			
			lien.setRole(roleNoun);
			lien.setClassId(idClasseRef);
		
			if (association.getStart()==null)
				association.setStart(lien);
			else
				association.setEnd(lien);
			
			associations.add(association);
			
			child=docProxy.getNextSibling(child);
			if (child==null) hasNext=false;
			
			
			

		}
		return associations;
		
	}
	
	/**
	 * retourne l'association qui correspond  la gnelizration pour le node Node pass en paramtre
	 * @param node le node concern
	 * @param docProxy le proxy sur l'objet document
	 * @return l'association qui correspond  la gnelizration pour le node Node pass en paramtre
	 */
	private Association getGeneralization(Node element, DocumentProxy2_1 docProxy){
		Association generalization=null;
		Node child=docProxy.getFirstChild(element);
		boolean hasNext=true;
		if (child==null) hasNext=false;
		while(hasNext){
			if (child.getNodeName().compareTo("generalization")!=0){
				child=docProxy.getNextSibling(child);
				if (child==null) hasNext=false;
				continue;
			}
			
			
			String general=docProxy.getAttributValue(child,"general");
			String id=docProxy.getAttributValue(child,"xmi:id");
			
			generalization=new Association();
			generalization.setLinkType(Association.HERITAGE);
			generalization.setId(id);
			Link lienMere=new Link();
			lienMere.setClassId(general);
			generalization.setStart(lienMere);
			break;
			

		}
		return generalization;
		
	}
	
	
	private void positionnerAssociationsDesClasses(String classPackage){
		//Parcourir les associations pour bien fixer les classes associations
		//et les heritages
		InfosModel infosModelTemp=new InfosModel();
		infosModelTemp.setAssociations(this.getAssociations());
		infosModelTemp.setClasses(this.getClasses());
		Enumeration<Association> enumeration=this.getAssociations().elements();
		while (enumeration.hasMoreElements()){
			Association association=enumeration.nextElement();					
			if (association.getLinkType().compareTo(Association.CLASSE_ASSOCIATION)==0){
				Classe classeLiee=(Classe)this.getClasses().get(association.getClassLinked().getId());
				classeLiee.setClassPackage(classPackage);
				ITransformer transformer=new ClassAssociationTransformer();
				transformer.transform(infosModelTemp, association);				
				classeLiee.getAssociations().remove(association);
			}else if (association.getLinkType().compareTo(Association.HERITAGE)==0){
				ITransformer transformer=new InheritanceTransformer();
				transformer.transform(infosModelTemp, association);

			}
		}
	}
	
	public Hashtable<String,Classe> getClasses() {
		return classes;
	}
	public void setClasses(Hashtable<String,Classe> classes) {
		this.classes = classes;
	}
	public Hashtable<String,Association> getAssociations() {
		return associations;
	}
	public void setAssociations(Hashtable<String,Association> associations) {
		this.associations = associations;
	}
	public Hashtable<String,String> getTypes() {
		return types;
	}
	public void setTypes(Hashtable<String,String> types) {
		this.types = types;
	}
	
	

}