/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package oued.persistance.infosmodelgenerator.transformer;


import oued.persistance.GeneratorControl;
import oued.persistance.classconcepts.Association;
import oued.persistance.classconcepts.Classe;
import oued.persistance.classconcepts.InfosModel;
import oued.persistance.classconcepts.Link;
import oued.persistance.classconcepts.Multiplicity;

/**
 * La classe {@link ClassAssociationTransformer} permet de transformer selon le besoin la facon
 *  dont sera gnre une classe association en hibernate. Si une classe association ne comporte
 *  pas elle mme des association, elle est gnre en compositeElement, par contre si elle contient
 *  des association, elle sera considre comme une classe a part entire: <br>
 * @author OUEDRAOGO Moussa
 * @since 08-05-2007
 *
 */
public class ClassAssociationTransformer implements ITransformer {

	public void transform(InfosModel infosModel,Association association) {
		Classe classeLiee=(Classe)infosModel.getClasses().get(association.getClassLinked().getId());
		Association associationRef=(Association)infosModel.getAssociations().get(association.getIdAssocRef());
		association.setStart(associationRef.getStart());
		association.setEnd(associationRef.getEnd());
		if (classeLiee.getAssociations().size()>0){//L'association a des relations
			Association ass_1=this.createAssociation(associationRef.getId()+"_1");
			ass_1.getStart().setClassId(classeLiee.getId());
			ass_1.getEnd().setClassId(association.getEnd().getClassId());
			
			Association ass_2=this.createAssociation(associationRef.getId()+"_2");
			ass_2.getStart().setClassId(classeLiee.getId());
			ass_2.getEnd().setClassId(association.getStart().getClassId());
			
			classeLiee.getAssociations().add(ass_1);
			classeLiee.getAssociations().add(ass_2);
			classeLiee.setEstClasseAssociation(false);
			infosModel.getAssociations().put(ass_1.getId(), ass_1);
			infosModel.getAssociations().put(ass_2.getId(), ass_2);
			
		}else{
			association.setClassLinked(classeLiee);		
			
			classeLiee.setEstClasseAssociation(true);
//			classeLiee.setClassPackage(classPackage);
			Classe classeDebut=(Classe)infosModel.getClasses().get(associationRef.getStart().getClassId());
			classeDebut.getAssociations().add(association);
			classeDebut.getAssociations().remove(associationRef);
			Classe classeFin=(Classe)infosModel.getClasses().get(associationRef.getEnd().getClassId());
			classeFin.getAssociations().add(association);
			classeFin.getAssociations().remove(associationRef);
		}
		
	}

	
	private Association createAssociation(String id){
		Association association_1=new Association();
		association_1.setId(id);
		
		Link start=new Link();
		start.setRole("");
		Multiplicity mult_1=new Multiplicity();
		mult_1.setCardMax("*");
		mult_1.setCardMin("*");
		start.setMultiplicity(mult_1);
		association_1.setStart(start);
		
		Link end=new Link();
		end.setRole("");
		Multiplicity mult_2=new Multiplicity();
		mult_2.setCardMax("1");
		mult_2.setCardMin("1");
		end.setMultiplicity(mult_2);
		association_1.setEnd(end);
		association_1.setLinkType(Association.SIMPLE_ASSOCIATION);
		
		
		return association_1;
	}

	

}
