/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance.mappingxml;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import oued.persistance.PersistanceZeroException;
import oued.persistance.classconcepts.Classe;
import oued.persistance.configuration.ConfigurationBuildException;
import oued.persistance.configuration.Configurator;
import oued.persistance.configuration.PersistanceZeroConfigurator;
import oued.persistance.utils.UtilityClassFunction;

/**
 * Cette classe per la gnration du fichier de configuration hibernate
 * @author OUEDRAOGO Moussa
 * @since 15-04-2007
 *
 */
public class HibernateConfigurationFile extends XmlFile {
	private Hashtable<String,Classe> classes;
	private String packageClasse;
	private String repertoirePersitanceZero;
	private String repertoireSources;
	private PersistanceZeroConfigurator mappingConfigurator;
	
	/**
	 * Le constructeur personnalis
	 * @param packageClasse le package des diffrentes sources
	 * @param classes l'ensemble des classes du projet
	 * @param repertoirePersitanceZero le repertoire de travail
	 * @param repertoireSources le repertoire des sources,relatif au repertoire de travail
	 */
	public HibernateConfigurationFile(String packageClasse,Hashtable<String,Classe> classes,String repertoirePersitanceZero,String repertoireSources) {
		this.classes = classes;
		this.packageClasse=packageClasse;
		this.repertoirePersitanceZero=repertoirePersitanceZero;
		this.repertoireSources=repertoireSources;
	}

	/**
	 * gnre le fichier de configuration hibernate
	 */
	public void genererFichierConfiguration() throws PersistanceZeroException {
		DocumentBuilderFactory fabrique = DocumentBuilderFactory.newInstance();
		DocumentBuilder constructeur=null;
		try {
			constructeur = fabrique.newDocumentBuilder();
		} catch (ParserConfigurationException e1) {
			throw new PersistanceZeroException("Impossible de crer le fichier de configuration hibernate.hbm.xml."+e1.getMessage());
		}
		Document document = constructeur.newDocument();
		Element racine=null;
		try {
			constructeur = fabrique.newDocumentBuilder();
			racine=document.createElement("hibernate-configuration");
			Element sessionFactory=document.createElement("session-factory");
			/**
			 * Definir les configuration personnalise
			 */
			if (mappingConfigurator!=null){
				/*Recuperer la configuration personnalise  pour l'attribut courant
				 * et les appliquer
				 */ 
				Iterator<Configurator>it=mappingConfigurator.getGlobalsConfigurators().iterator();
				while (it.hasNext()){
					Configurator configuration=it.next();
					try {
						if (configuration!=null){
							configuration.buildProprietes();														
							Element element=this.createProperty(document,"property","name",configuration.getEltsParametres().get(0), configuration.getValeursParametres().get(0));
							sessionFactory.appendChild(element);
						}
					} catch (ConfigurationBuildException e) {
						System.out.println("Une erreur lors de la prise en compte des paramtre personnalis pour le context hibernate.hbm.xml");
						System.out.println(e.getErreur());
						e.printStackTrace();
					}
				}
				
				
			}	
				
			
			Enumeration<Classe> it=classes.elements();
			while (it.hasMoreElements()){				
				Classe classe=(Classe)it.nextElement();
				if ( (!classe.isChild())
						&& (!classe.isEstClasseAssociation())
					){
					Element mapping=this.createProperty(document,"mapping","resource",UtilityClassFunction.replacePointBySlash(this.packageClasse)+"\\"+UtilityClassFunction.firstLetterUpperCase(classe.getNom()+".hbm.xml"), null);
					sessionFactory.appendChild(mapping);
				}
				
			}
			
			racine.appendChild(sessionFactory);
			
		} catch (ParserConfigurationException e) {
			throw new PersistanceZeroException("Impossible de crer le fichier de configuration hibernate.hbm.xml."+e.getMessage());
		}
		
		document.appendChild(racine);
		String nomCompletFichier="";
		if (this.repertoireSources!=null && this.repertoireSources.trim().compareTo("")!=0)
			nomCompletFichier=repertoirePersitanceZero+"\\"+this.repertoireSources+"\\hibernate.cfg.xml";
		else
			nomCompletFichier=this.repertoirePersitanceZero+"\\hibernate.cfg.xml";
		
		this.setNomFichier(nomCompletFichier);
		this.setDoctypePublic("-//Hibernate/Hibernate Configuration DTD 3.0//EN");
		this.setDoctypeSystem("http://hibernate.sourceforge.net/hibernate-configuration-3.0.dtd");
		this.setDocument(document);
		try {
			try {
				this.transformXml();
			} catch (ParserConfigurationException e) {
				throw new PersistanceZeroException("Impossible de crer le fichier de configuration hibernate.hbm.xml."+e.getMessage());
			}
		} catch (TransformerException e) {
			throw new PersistanceZeroException("Impossible de crer le fichier de configuration hibernate.hbm.xml."+e.getMessage());
		}
	}
	/**
	 * retourne la liste des classe
	 * @return la liste des classe
	 */
	public Hashtable<String,Classe> getClasses() {
		return classes;
	}
	
	/**
	 * modifie la liste des classe
	 * @param  classes la liste des classe
	 */
	public void setClasses(Hashtable<String,Classe> classes) {
		this.classes = classes;
	}
	public String getPackageClasse() {
		return packageClasse;
	}

	/**
	 * modifie le package des classes
	 * @param packageClasse la liste des classe
	 */
	public void setPackageClasse(String packageClasse) {
		this.packageClasse = packageClasse;
	}
	
	
	/**
	 * Cree un propriete du fichier de configuration d'hibernate
	 * @param document le document
	 * @param nomBalise la balise de la propriete
	 * @param nomAttribut le nom de l'attribut
	 * @param valeurNom la valeur de l'attribut
	 * @param contenu le contenu de l'attribut
	 */
	private Element createProperty(Document document,String nomBalise,String nomAttribut,String valeurNom,String contenu){
		Element property1=document.createElement(nomBalise);
		property1.setAttribute(nomAttribut,valeurNom);
		
		if (contenu!=null)
			property1.setTextContent(contenu);
		
		return property1;
	}

	public PersistanceZeroConfigurator getMappingConfigurator() {
		return mappingConfigurator;
	}

	public void setMappingConfigurator(PersistanceZeroConfigurator mappingConfigurator) {
		this.mappingConfigurator = mappingConfigurator;
	}

	
	
	

	
}
