/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance.mappingxml;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import oued.persistance.classconcepts.Association;
import oued.persistance.classconcepts.Classe;
import oued.persistance.classconcepts.InfosModel;
import oued.persistance.classconcepts.Link;
import oued.persistance.classconcepts.SimpleProperty;
import oued.persistance.classconcepts.DataType;
import oued.persistance.configuration.ClassConfigurator;
import oued.persistance.configuration.ConfigurationBuildException;
import oued.persistance.configuration.Configurator;
import oued.persistance.configuration.PersistanceZeroConfigurator;
import oued.persistance.utils.UtilityClassFunction;

/**
 * Cette classe reprsente le mapping cfg d'une classe
* @author ouedmouss
 * @author OUEDRAOGO Moussa
 * @since 15-04-2007
 *
 */

public class HibernateMappingFile  extends XmlFile{
	private List<String> tempNames=new ArrayList<String>();
	private List<Configurator> listeMultiAttrConfiguration;
	private List<AttributKeeper> listeAttributKept;
	DocumentBuilder constructor;
	private Classe classe;
	private InfosModel infosModel;
	private ClassConfigurator classConfigurator;
	
	/**
	 * Le constructeur personnalis
	 * @param infosModel le infosModele qui contient toutes les ressources
	 * @param classe la classe du hibernate mapping file
	 */
	public HibernateMappingFile(InfosModel infosModel,Classe classe) {
		listeAttributKept=new ArrayList<AttributKeeper>();
		listeMultiAttrConfiguration=new ArrayList<Configurator>();
		this.classe=classe;
		this.infosModel=infosModel;
		
	}

	/**
	 * Creer le mapping d'une classe
	 */
	public void createMappingXML(String workingDir,String srcDir) throws CreateMappingFileException{
		if (tempNames==null) tempNames=new ArrayList<String>();
		else tempNames.clear();
		try{
			Document document = constructor.newDocument();
			this.setDocument(document);
			
			//L'element racine
			Element racine=document.createElement("hibernate-mapping");
			racine.setAttribute("package",this.getClasse().getClassPackage());
			
			/**
			 * Recuperer toutes les configurations de contexte
			 */
			if (classConfigurator!=null){
				List<String> contexts=new ArrayList<String>();
				contexts.add(PersistanceZeroConfigurator.NATURAL_ID);
				contexts.add(PersistanceZeroConfigurator.COMPONENT_CONTEXT);
				contexts.add(PersistanceZeroConfigurator.PROPERTIES);
				listeMultiAttrConfiguration=classConfigurator.getAllContextConfiguration(contexts);
			}
			
			/*Lie la racine au corps du mapping. Pour la premiere fois on lance createCorps avec un parametre null (3 eme parametre)
			//car une classe a cet stade ne peut avoir une classe mere.
			 * Souvenez vous, dans la methode genererXML, nous n'appelons createMappingXML que pour les classes qui ne sont pas filles et qui ne 
			 * sont pas une classe association. Une classe qui n'est pas fille n'a donc pas de mere.
			*/
			tempNames.clear();
			Element corps=this.createCorps(classe,document,null);
			racine.appendChild(corps);
			document.appendChild(racine);
			
			//Le nom complet du fichier xml a creer. Ce nom est compose du nom de package, suivi du nom de la classe et d'une extension .hml.xml
			//Exemple, pour une classe de nom Etudiant dans le package oued.pojo aurra comme nom
			//oued.pojo.Etudiant.hbm.xml
			String nomCompletFichierXML;
			if (srcDir!=null && srcDir.trim().compareTo("")!=0)
				nomCompletFichierXML=workingDir+"\\"+srcDir+"\\"+UtilityClassFunction.replacePointBySlash(this.classe.getClassPackage())+"\\"+classe.getNom()+".hbm.xml";
			else
				nomCompletFichierXML=workingDir+"\\"+UtilityClassFunction.replacePointBySlash(this.classe.getClassPackage())+"\\"+classe.getNom()+".hbm.xml";
			
			//Effectuer la transformation de la classe en document xml
			this.setNomFichier(nomCompletFichierXML);
			this.setDoctypePublic("-//Hibernate/Hibernate Mapping DTD 3.0//EN");
			this.setDoctypeSystem("http://hibernate.sourceforge.net/hibernate-mapping-3.0.dtd");
			this.setIndent(true);
			this.transformXml();
		}catch(Exception e){
			throw new CreateMappingFileException(e.getMessage());
		}
				
	}
	
	/***
	 * retourne la liste des Configurations pour les contexts tels que PROPERTIES, COMPONENT, NATURAL_ID.
	 * Pour ces contextes on doit garders les diffrents Element des attributs afin de les mettres dans un properties
	 * @return la liste des Configurations pour les contexts tels que PROPERTIES, COMPONENT, NATURAL_ID.
	 */
	public List<Configurator> getListeMultiAttrConfiguration() {
		return listeMultiAttrConfiguration;
	}

	/***
	 * modifie la liste des Configurations pour les contexts tels que PROPERTIES, COMPONENT, NATURAL_ID.
	 * Pour ces contextes on doit garders les diffrents Element des attributs afin de les mettres dans un properties
	 * @param listeMultiAttrConfiguration  la nouvelle liste des Configurations pour les contexts tels que PROPERTIES, COMPONENT, NATURAL_ID.
	 */
	public void setListeMultiAttrConfiguration(List<Configurator> listeMultiAttrConfiguration) {
		this.listeMultiAttrConfiguration = listeMultiAttrConfiguration;
	}

	/***
	 * retourne la classe pour laquelle on doit gnrer le fichier de configuration
	 * @return la classe pour laquelle on doit gnrer le fichier de configuration
	 */
	public Classe getClasse() {
		return classe;
	}

	/***
	 * modifie la classe pour laquelle on doit gnrer le fichier de configuration
	 * @param classe la  nouvelle classe pour laquelle on doit gnrer le fichier de configuration
	 */
	public void setClasse(Classe classe) {
		this.classe = classe;
	}

	/***
	 * retourne le  DocumentBuilder  pour la cration de l'objet document
	 * @return le  DocumentBuilder  pour la cration de l'objet document
	 */
	public DocumentBuilder getConstructeur() {
		return constructor;
	}

	/***
	 * modifie le  DocumentBuilder  pour la cration de l'objet document
	 * @param constructeur le nouveau DocumentBuilder  pour la cration de l'objet document
	 */
	public void setConstructeur(DocumentBuilder constructeur) {
		this.constructor = constructeur;
	}

	/***
	 * retorune le  ClasseConfigurator de cette classe
	 * @return le  ClasseConfigurator de cette classe
	 */
	public ClassConfigurator getClasseConfigurator() {
		return classConfigurator;
	}

	/***
	 * modifie le  ClasseConfigurator de cette classe
	 * @param classeConfigurator le nouveau  ClasseConfigurator de cette classe
	 */
	public void setClasseConfigurator(ClassConfigurator classeConfigurator) {
		this.classConfigurator = classeConfigurator;
	}
	
	/***
	 * applique les configuration personnelles de l'utilisateur pour un context ou pour un attribut particulier
	 * @param element Element concerne 
	 * @param isContextConfiguration 
	 * @param le filtre de recherche de l'objet {@link Configurator}
	 */
	

	/**
	 * Methode permet de creer le corps du fichier xml. C'est a dire toute la partie se trouvant dans la balise
	 * <class name="..." >
	 * </class>
	 * 
	 * @param classe le nom de la classe a creer
	 * @param document l'objet document qui sert a la creation du document xml
	 * @param classeMere la classe mere de la classe s'il y en a
	 * @return
	 */
	private Element createCorps(Classe classe, Document document,Classe classeMere ){
		
		//boolean joins=false;
		/*map_corps contient les elements de corps de la classe. C'est a dire tout ce qui se trouve dans
		*  <class>
		*  </class> 
		*  dans le cas d'une classe normale
		 * 
		 * ou
		 *  <joined-subclass>
		 *  </joined-subclass> dans le cas d'une classe fille
		*/
		Element map_corps=null;
		
		
		if (classeMere!=null){
			//Si ce cas est vrai c'est que la fonction a ete appelee de facon recursive. C'est a dire depuis
			//Une premiere methode createCorps. C'est donc un element d'heritage...
			//La classe est donc une classe fille
			tempNames.clear();
			map_corps=this.createJoin(classe,document,classeMere);
			//joins=true;
		}else{
			//C'est l'appel normal. On a une classe qui n'est pas fille
			map_corps=this.createClass(classe,document);
		}
		
		
//		Creer les attributs de la classe
		map_corps=this.createAttributs(classe,document,map_corps);
//		Creer le mapping pour les associations avec cette classe
		map_corps=this.createAssociations(classe,document,map_corps);
		
		for(int i=0;i<listeMultiAttrConfiguration.size();i++){
			Configurator configuration=listeMultiAttrConfiguration.get(i);
			String nomMulit=null;
			if (configuration.getMappingContext().getContext().compareTo(PersistanceZeroConfigurator.PROPERTIES)==0)
				nomMulit="properties";
			else if (configuration.getMappingContext().getContext().compareTo(PersistanceZeroConfigurator.COMPONENT_CONTEXT)==0)
				nomMulit="component";
			else if (configuration.getMappingContext().getContext().compareTo(PersistanceZeroConfigurator.COMPONENT_CONTEXT)==0)
				nomMulit="natural-id";
			Element multiAttribut=document.createElement(nomMulit);		
			try{
				if (configuration!=null){
					configuration.buildProprietes();	
					for(int j=0;j<configuration.getEltsParametres().size();j++){
						multiAttribut.setAttribute(configuration.getEltsParametres().get(j),configuration.getValeursParametres().get(j));
					}
				}
			}catch(ConfigurationBuildException e){
				
			}
			for(int j=0;j<listeAttributKept.size();j++){
				AttributKeeper saver=listeAttributKept.get(j);
				if (configuration.contientAttribut(saver.getNomAttribut())){					
					multiAttribut.appendChild(saver.getElement());
				}
			}
			map_corps.appendChild(multiAttribut);
		}
		return map_corps;
		
	}
	
	/**
	 * Creer la section classe du mapping xml
	 * @param classe la classe pour laqelle on doit creer cette section
	 * @param document
	 * @return
	 */
	private Element createClass(Classe classe, Document document){
		Element classeInfos = document.createElement("class");
		classeInfos.setAttribute("name",classe.getNom());
		classeInfos.setAttribute("table",classe.getNom());
		
		Element id = document.createElement("id");
		id.setAttribute("column","ID_"+classe.getNom());
		id.setAttribute("name","id");			
		
		Element generator = document.createElement("generator");
		generator.setAttribute("class","native");
		
		this.appliquerConfigurationPersonnalisee(id, true,PersistanceZeroConfigurator.ID);	
		this.appliquerConfigurationPersonnalisee(classeInfos, true, PersistanceZeroConfigurator.CLASS);			
		classeInfos.appendChild(id);
		id.appendChild(generator);
		
		return classeInfos;
	}
	
	
	/**
	 * Creer la section joined-subclass du mapping xml
	 * @param classe la classe pour laqelle on doit creer cette section
	 * @param document le document servant a creer le fichier xml
	 * @return
	 */
	private Element createJoin(Classe classe, Document document,Classe classeMere){
		Element joined_subclass=document.createElement("joined-subclass");
			joined_subclass.setAttribute("name",classe.getNom());
			joined_subclass.setAttribute("table",classe.getNom());
			Element key=document.createElement("key");
			key.setAttribute("column","ID_"+classe.getNom());
			joined_subclass.appendChild(key);
			return joined_subclass;
	}
	
	
	/**
	 * Creer les attributs xml d'une classe
	 * @param classe la classe pour laquelle on doit creer les attributs
	 * @param document l'objet document a utiliser pour la creation
	 * @param map_corps le corps xml de classe dans laquelle on va inserer les attributs
	 * @return
	 */
	private Element createAttributs(Classe classe,Document document,Element map_corps){
		/**
		 * Prendre en compte le versionning
		 */
		Element version=document.createElement("version");
		this.appliquerConfigurationPersonnalisee(version, true,PersistanceZeroConfigurator.VERSION);			
		
		//Generer les attributs de type simple
		Iterator<SimpleProperty> itAttrs=classe.getProprietesSimples().iterator();
		while (itAttrs.hasNext()){
			SimpleProperty attr=(SimpleProperty)itAttrs.next();
			Element attribut=document.createElement("property");			
			attribut.setAttribute("name",attr.getName());
			tempNames.add(attr.getName());
			String type=null;
			
			
			if (type!=null)
				attribut.setAttribute("type",(String)infosModel.getTypes().get(attr.getIdType()));
			else
				attribut.setAttribute("type",DataType.STRING);
			attribut.setAttribute("column",attr.getName());
			attribut.setAttribute("node","@"+attr.getName());
			
			this.appliquerConfigurationPersonnalisee(attribut, false,attr.getName());				
			
			boolean estDedans=false;
			for(int i=0;i<listeMultiAttrConfiguration.size();i++){
				Configurator conf=listeMultiAttrConfiguration.get(i);
				if (conf.contientAttribut(attr.getName())){
					estDedans=true;
					break;
				}
				
			}
			
			if (!estDedans)
				map_corps.appendChild(attribut);
			else{
				AttributKeeper conservateur=new AttributKeeper(attribut,"Attribut Simple",attr.getName());				
				listeAttributKept.add(conservateur);
			}
			
		}
		return map_corps;
	}
	
	
	/**
	 * Creer les association xml d'une classe
	 * @param classe la classe pour laquelle on doit creer les attributs
	 * @param document l'objet document a utiliser pour la creation
	 * @param map_corps le corps xml de classe dans laquelle on va inserer les associations
	 * @return
	 */
	private Element createAssociations(Classe classe,Document document,Element map_corps){
		//Element map_association=null;
		Element map_heritage=null;
		//Element map_classAssociation=null;
		
		//Parcourir l'ensemble des associations de la classe afin de les creer
		Iterator<Association> it =classe.getAssociations().iterator();
		while (it.hasNext()){
			Association association=(Association)it.next();		
			//Si c'est une  association simle, une agregation ou une composition 
			if ( (association.getLinkType().compareTo(Association.SIMPLE_ASSOCIATION)==0)
					|| (association.getLinkType().compareTo(Association.AGREGATION)==0)
					|| (association.getLinkType().compareTo(Association.COMPOSITION)==0)){
				this.createAssociationHabituelle(classe,document,map_corps,association);
				
				
				
			}
			
			//Si c'est une classe association
			else if (association.getLinkType().compareTo(Association.CLASSE_ASSOCIATION)==0){
				this.createClasseAssociation(classe,document,map_corps,association);
			}
			
			//Si c'est une relation d'heritage.
			
			else if (UtilityClassFunction.isMotherInAssociation(classe,association) ){
				Classe fille=UtilityClassFunction.childClass(infosModel.getClasses(),association);
				if (fille.getId().compareTo(classe.getId())!=0){
					
					map_heritage=this.createCorps(fille,document,classe);
					map_corps.appendChild(map_heritage);
					
				}
			}
		
		}
		return map_corps;
	}
	
	/**
	 * Methode pour la creation d'association simple, d'agregation ou de composition
	 * @param classe
	 * @param document
	 * @param association
	 * @return
	 */
	private void createAssociationHabituelle(Classe classe, Document document,Element map_corps, Association association){
		NameResolver nameResolver=new NameResolver(tempNames);
		//Recuperer l'autre bout de l'association
		Classe autreClasse=UtilityClassFunction.otherAssociationClass(infosModel.getClasses(),classe, association);
		Link lienAutreClasse= UtilityClassFunction.classLink(autreClasse, association);
		Link lienClasseCourante= UtilityClassFunction.otherClassLink(autreClasse, association);
//		Gerer le cas ou une classe a plusieurs associations avec une autre classe
		//On ne doit pas avoir le meme nom d'attribut
			String name="";
			if (lienAutreClasse.getRole()!=null & lienAutreClasse.getRole().trim().compareTo("")!=0){
				name=lienAutreClasse.getRole();
			}
			else
				name=autreClasse.getNom();
		if (lienAutreClasse.getClassId().compareTo(autreClasse.getId())!=0){
			lienAutreClasse=new Link();
			lienClasseCourante=new Link();
			lienAutreClasse=association.getStart();
			lienClasseCourante=association.getEnd();
			
		}
		
		if (lienAutreClasse.getMultiplicity().isOne_One()){
			Element attributComplexe=document.createElement("many-to-one");		
			name=nameResolver.retrieveName(lienAutreClasse, lienClasseCourante, autreClasse);
			attributComplexe.setAttribute("name",name);						
			attributComplexe.setAttribute("column","ID_"+name);
			attributComplexe.setAttribute("class",autreClasse.getNom());
			tempNames.add(name);
			
			this.appliquerConfigurationPersonnalisee(attributComplexe, false,name);				
			
			boolean estDedans=false;
			for(int i=0;i<listeMultiAttrConfiguration.size();i++){
				Configurator conf=listeMultiAttrConfiguration.get(i);
				if (conf.contientAttribut(name)){
					estDedans=true;
					break;
				}
				
			}
			
			if (!estDedans){
				map_corps.insertBefore(attributComplexe,map_corps.getFirstChild().getNextSibling());
			}
			else{
				AttributKeeper conservateur=new AttributKeeper(attributComplexe,"Attribut Simple",UtilityClassFunction.firstLetterLowerCase(name));			
				listeAttributKept.add(conservateur);
			}			
		}	
		else if (lienAutreClasse.getMultiplicity().isManyToMany()){
			if (lienClasseCourante.getMultiplicity().isManyToMany()){	
					name=nameResolver.retrieveName(lienAutreClasse, lienClasseCourante, autreClasse);
					Element set=document.createElement("set");
					Classe debut=(Classe)infosModel.getClasses().get(association.getStart().getClassId());
					Classe fin=(Classe)infosModel.getClasses().get(association.getEnd().getClassId());										
					set.setAttribute("name",name);					
					set.setAttribute("table",debut.getNom()+fin.getNom()+nameResolver.getDiff()+association.getName());
					Element key=document.createElement("key");
					key.setAttribute("column","ID_"+name);
					Element one_to_many=document.createElement("many-to-many");					
					one_to_many.setAttribute("class",autreClasse.getNom());
					
					set.appendChild(key);
					set.appendChild(one_to_many);
					tempNames.add(name);
					this.appliquerConfigurationPersonnalisee(set, false,name);					
					map_corps.insertBefore(set,map_corps.getFirstChild().getNextSibling());
			}else{				
					name=nameResolver.retrieveName(lienAutreClasse, lienClasseCourante, autreClasse);
				String incr="";
				while (tempNames.contains(name)){
					incr="1"+incr;
					name=name+incr;
				}
				Element set2=document.createElement("set");
				set2.setAttribute("name",name);				
				Element key2=document.createElement("key");
				key2.setAttribute("column","ID_"+classe.getNom());
				Element one_to_many2=document.createElement("one-to-many");
								
				one_to_many2.setAttribute("class",autreClasse.getNom());
				set2.appendChild(key2);
				set2.appendChild(one_to_many2);
				tempNames.add(name);
				this.appliquerConfigurationPersonnalisee(set2, false, name);							
				map_corps.insertBefore(set2,map_corps.getFirstChild().getNextSibling());
			}

		}
		
	}
	
	/**
	 * Creer la section classe association dans le fichier xml de la classe principale
	 * @param classe la classe courante
	 * @param document le document
	 * @param map_corps Element corps du fichier xml
	 * @param association l'association courante
	 */
	private void createClasseAssociation(Classe classe, Document document,Element map_corps, Association association){	
		Classe classeLiee=association.getClassLinked();
		Classe tempClass=(Classe)infosModel.getClasses().get(association.getStart().getClassId());
		int index=classeLiee.getNom().indexOf(tempClass.getNom());
		Classe classePrincipale;
		Classe classeSecondaire;
		if (index==0){
			classePrincipale=tempClass;
			classeSecondaire=(Classe)infosModel.getClasses().get(association.getEnd().getClassId());
		}else{
			classeSecondaire=tempClass;
			classePrincipale=(Classe)infosModel.getClasses().get(association.getEnd().getClassId());
		}
		if (classePrincipale!=classe)
			return ;
		Element set=document.createElement("bag");
		set.setAttribute("name","liste"+classeLiee.getNom());
		set.setAttribute("table",classeLiee.getNom());
		
		Element key=document.createElement("key");
		key.setAttribute("column","ID_"+classe.getNom());
		
		Element composite_element=document.createElement("composite-element");
		composite_element.setAttribute("class",classeLiee.getNom());
		
		Element many_to_one=document.createElement("many-to-one");
		many_to_one.setAttribute("name",classeSecondaire.getNom());
		many_to_one.setAttribute("class",classeSecondaire.getNom());
		many_to_one.setAttribute("column","ID_"+classeSecondaire.getNom());
		
		set.appendChild(key);
		set.appendChild(composite_element);
		composite_element.appendChild(many_to_one);
		//Append des attributs
//		Generer les attributs de type simple
		Iterator<SimpleProperty> itAttrs=classeLiee.getProprietesSimples().iterator();
		while (itAttrs.hasNext()){
			SimpleProperty attr=(SimpleProperty)itAttrs.next();
			Element attribut=document.createElement("property");
			attribut.setAttribute("name",attr.getName());
			
			String type=null;
			if (attr.getIdType()!=null)
			   type=(String)infosModel.getTypes().get(attr.getIdType());
			if (type!=null)
				attribut.setAttribute("type",(String)infosModel.getTypes().get(attr.getIdType()));
			attribut.setAttribute("column",attr.getName());
			composite_element.appendChild(attribut);
			
		}
		
		this.appliquerConfigurationPersonnalisee(set, false,"liste"+classeLiee.getNom());				
		if (set!=null){			    
			map_corps.insertBefore(set,map_corps.getFirstChild().getNextSibling());
		}		
	}
	
	
	private void appliquerConfigurationPersonnalisee(Element element,boolean isContextConfiguration, String filtreConfigurator){
		if (classConfigurator!=null){
			/*Recuperer la configuration personnalise  pour l'attribut courant
			 * et les appliquer
			 */ 
			Configurator configuration=null;
			if (isContextConfiguration)
				configuration=classConfigurator.getFirstContextConfiguration(filtreConfigurator);
			else
				configuration=classConfigurator.getAttributConfiguration(filtreConfigurator);
			try {
				if (configuration!=null){
					configuration.buildProprietes();	
					for(int i=0;i<configuration.getEltsParametres().size();i++){
						element.setAttribute(configuration.getEltsParametres().get(i),configuration.getValeursParametres().get(i));
					}
				}
			} catch (ConfigurationBuildException e) {
				System.out.println("Une erreur lors de la prise en compte des paramtre personnalis pour le context "+PersistanceZeroConfigurator.ID+"de la classe "+classe.getNom());
				System.out.println(e.getErreur());
				e.printStackTrace();
			}
			
			
		}	
	}
	
	private String retrieveDifferentName(String name){
		String incr="";
		String nomTemporaire="";
		while (tempNames.contains(name)){
			incr="1"+incr;
			nomTemporaire=nomTemporaire+incr;
		}
		return nomTemporaire;
	}
}
