/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance.mappingxml;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;

/**
 * Cette classe reprsente un fichier XMI
 * @author OUEDRAOGO Moussa
 * @since 04-03-2006
 *
 */
public class XmlFile {
	/**
	 * L'objet document qui va servir  la cration du fichier xml
	 */
	private Document document;
	
	/**
	 *Le nom complet du fichier xml 
	 */
	private String nomFichier;
	
	/**
	 * L'encodage du fichier xml. Par dfaut vaut ISO-8859-1
	 */
	private String encodage="ISO-8859-1";
	
	/**
	 *doit on indenter ou non le fichier 
	 */
	private boolean indenter=true;
	
	/**
	 * Le doctype publique
	 */
	private String doctypePublique="";
	
	/**
	 * le doctype systme
	 */
	private String doctypeSysteme="";
	
	/**
	 * transorme le fichier xml en fichier physique sur le disque
	 * @throws ParserConfigurationException exception lance si un problme survient
	 * @throws TransformerException exception lance si un problme survient
	 */
	public void transformXml() throws ParserConfigurationException, TransformerException {
            // Cration de la source DOM
            Source source = new DOMSource(document);
            // Cration du fichier de sortie
            Result resultat = new StreamResult(nomFichier);
            // Configuration du transformer
            TransformerFactory fabrique = TransformerFactory.newInstance();
            Transformer transformer = fabrique.newTransformer();
            transformer.setOutputProperty(OutputKeys.ENCODING, encodage);
            String ind=indenter==true ? "yes" :"false";
            transformer.setOutputProperty(OutputKeys.INDENT, ind);           
//            transformer.setOutputProperty(OutputKeys.DOCTYPE_PUBLIC,"-//Hibernate/Hibernate Mapping DTD 3.0//EN");
//            transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM,"http://hibernate.sourceforge.net/hibernate-configuration-3.0.dtd");   
            transformer.setOutputProperty(OutputKeys.DOCTYPE_PUBLIC,doctypePublique);
            transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM,doctypeSysteme);
            transformer.transform(source, resultat);
      
    }
	
	/**
	 * retourne l'objet document qui va servir  la cration du fichier xml
	 * @return l'objet document qui va servir  la cration du fichier xml
	 */
	public Document getDocument() {
		return document;
	}
	
	/**
	 * modifie l'objet document qui va servir  la cration du fichier xml
	 * @param document  le nouvel objet document qui va servir  la cration du fichier xml
	 */
	public void setDocument(Document document) {
		this.document = document;
	}
	
	/**
	 * retourne l'encodage du fichier
	 * @return l'encodage du fichier
	 */
	public String getEncodage() {
		return encodage;
	}
	
	/**
	 * modifie l'encodage du fichier
	 * @param encodage l'encodage du fichier
	 */
	public void setEncodage(String encodage) {
		this.encodage = encodage;
	}
	
	/**
	 * doit on indenter le fichier xml ou non
	 * @return est que l'on doit on indenter le fichier xml ou non
	 */
	public boolean isIndenter() {
		return indenter;
	}
	/**
	 * modifie le fait que l'on doit indenter ou non le fichier xml
	 * @param indenter le nouvel statut de l'encodage
	 */
	public void setIndent(boolean indenter) {
		this.indenter = indenter;
	}
	/**
	 * retourne le doctype publique du fichier xml
	 * @return le doctype publique du fichier xml
	 */
	public String getDoctypePublique() {
		return doctypePublique;
	}
	
	/**
	 * modifie le doctype publique du fichier xml
	 * @param doctypePublique le doctype publique du fichier xml
	 */
	public void setDoctypePublic(String doctypePublique) {
		this.doctypePublique = doctypePublique;
	}
	
	/**
	 * retourne le doctype systme du fichier xml
	 * @return le doctype systme du fichier xml
	 */
	public String getDoctypeSysteme() {
		return doctypeSysteme;
	}
	/**
	 * modifie le doctype systme du fichier xml
	 * @param doctypeSysteme le doctype systme du fichier xml
	 */
	public void setDoctypeSystem(String doctypeSysteme) {
		this.doctypeSysteme = doctypeSysteme;
	}

	/**
	 * retourne le nom complet du fichier xml
	 * @return le nom complet du fichier xml
	 */
	
	public String getNomFichier() {
		return nomFichier;
	}

	/**
	 * modifie le nom complet du fichier xml
	 * @param nomFichier le nom complet du fichier xml
	 */
	public void setNomFichier(String nomFichier) {
		this.nomFichier = nomFichier;
	}
	
	
	
	

}
