/*
 * Copyright 2007  OUEDRAOGO Moussa.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package oued.persistance.utils;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import oued.persistance.classconcepts.Association;
import oued.persistance.classconcepts.Classe;
import oued.persistance.classconcepts.Link;
import oued.persistance.classconcepts.DataType;

/**
 * Cette classe fournit des fonctions utilitaires pour la gestion des classes et des chaines de
 * caractre
* @author OUEDRAOGO Moussa
 * @since 04-02-2007
 *
 */
public class UtilityClassFunction {
	/**
	 * Cette mthode permet dans remplacer dans une chaine de cararactre un . par un \
	 * @param chaine la chaine dans laquelle il faut remplacer
	 * @return la chaine chaine ayant ces caractre . remplac par \
	 */
	public static String replacePointBySlash(String chaine){
		return chaine.replace(".","\\");
	}
	
	/**
	 * Cette mthode permet dans remplacer dans une chaine de cararactre un \ par un .
	 * @param chaine la chaine dans laquelle il faut faire le remplacement
	 * @return la chaine chaine ayant ces caractre \ remplac par .
	 */
	public static String replaceSlashByPoint(String chaine){
		return chaine.replace("\\",".");
	}
	/**
	 * Cette mthode permet de convertir les types de donnes basiques saisies dans le fichier xmi par les types de donnes
	 * basique Java. Seulement les types de donnes suivant sont pris en charge : String, int, Date, Double
	 * @param type le type correspondant au type de donnes basique de notre fichier xmi
	 * @return le type java correct correspondant au type pass en paramtre
	 */
	public static String convertirWrappeType(String type){
		if (type.compareTo(DataType.STRING)==0) return "String";
		if (type.compareTo(DataType.INTEGER)==0) return "int";
		if (type.compareTo(DataType.INT)==0) return "int";
		if (type.compareTo(DataType.DATE)==0) return "Date";
		if (type.compareTo(DataType.DOUBLE)==0) return "Double";
		return type;
	}
	
	/**
	 * Cette mthode retourne la chaine passe en paramtre aprs avoir mis sa premire lettre en majuscule
	 * @param attribut l'attribut dont il faut mettre la premiere lettre en majuscule
	 * @return la chaine pass en paramtre avec son premier attribut en majuscule
	 */
	public static String firstLetterUpperCase(String attribut){
		return attribut.substring(0,1).toUpperCase()+attribut.substring(1);		
	}
	/**
	 * Cette mthode retourne la chaine passe en paramtre aprs avoir mis sa premire lettre en minuscule
	 * @param attribut l'attribut dont il faut mettre la premiere lettre en majuscule
	 * @return la chaine pass en paramtre avec son premier attribut en majuscule
	 */
	public static String firstLetterLowerCase(String attribut){
		return attribut.substring(0,1).toLowerCase()+attribut.substring(1);		
	}
	/**
	 * Cette mthode retourne la classe tant l'autre bout de l'association. C'est  dire la classe 
	 * se trouvant dans le lien autre que le lien de la classe classe passe en paramtre
	 * @param classes l'ensemble des classes
	 * @param classe la classe correspond  la classe d'un lien de l'association.
	 * @param association L'association entre les deux classes
	 * @return retourne la classe tant l'autre bout de l'association. 
	 */
	public static Classe otherAssociationClass(Hashtable<String,Classe> classes,Classe classe, Association association){
		if (association.getStart().getClassId().compareTo(classe.getId())==0)
			return (Classe)classes.get(association.getEnd().getClassId());
		else
			return (Classe)classes.get(association.getStart().getClassId());
	}
	
/**
 * Cette mthode retourne le lien de la classe passe en paramtre pour l'association
 * @param classe la classe pour laquelle on recherche le lien
 * @param association L'association qui dfinit les lien entre les classes
 * @return le lien de la classe passe en paramtre pour l'association passe ne paramtre
 */
	public static Link classLink(Classe classe,Association association){
		if (association.getStart().getClassId().compareTo(classe.getId())==0)
				return association.getStart();
		else
			return association.getEnd();
	}
	
	/**
	 * Cette mthode retourne le lien de la classe autre que la classe passe en paramtre pour l'association 
	 * @param classe la classe pour laquelle on recherche le lien de la  classe avec qui elle est en relation
	 * @param association L'association qui dfinit les lien entre les classes
	 * @return retourne le lien de la classe autre que la classe passe en paramtre pour l'association 
	 */
	public static Link otherClassLink(Classe classe,Association association){
		if (association.getStart().getClassId().compareTo(classe.getId())!=0)
				return association.getStart();
		else
			return association.getEnd();
	}
	

	/**
	 * Retourne la classe mre de la classe passe en paramtre
	 * @param classes l'ensemble des classe
	 * @param classe la classe pour laquelle on recherche la mre
	 * @return a classe mre de la classe passe en paramtre
	 */	
	public static Classe motherClass(Hashtable<String,Classe>  classes, Classe classe){
		Iterator<Association> it=classe.getAssociations().iterator();
		Classe classeMere=null;
		while (it.hasNext()){
			Association association=(Association)it.next();
			if (association.getLinkType().compareTo(Association.HERITAGE)==0){
				classeMere=(Classe)classes.get(association.getStart().getClassId());
				if (classeMere.getId().compareTo(classe.getId())==0) 
					classeMere=null;
			}
		}
		return classeMere;
	}
	
	/**
	 * Cette mthode retourne la classe dite secondaire pour la classe lie passe en paramtre
	 * Lorsque deux classes sont associes l'une  l'autre  travers une classe association,
	 * Si la classe association commence par le nom d'une des classes, alors cette classe est dite principale.
	 * L'autre classe est dite secondaire.
	 * Si par exemple on a les classe Etudiant et Matiere qui sont lie par une classe association de nom EtudiantCoefficient alors Etudiant
	 * est la classe principale. L'autre est donc secondaire.
	 * Si par contre cette rgle n'est pas respecte une des classe devient classe principale (cela de faon arbitraire)
	 * La particularit de cette mthode, c'est que l'on ne connait pas l'association qui relie les classes. On la recherche grace  la classe association
	 * @param classes l'ensemble des classes
	 * @return retourne la classe dite secondaire dans une classe association.
	 */
	public static Classe secondaryClassForLinkedClass(Hashtable<String,Association>  associations,Hashtable<String,Classe>  classes, Classe linkedClasse){
		Enumeration<Association> it=associations.elements();
		Classe classeSecondaire=null;
		while (it.hasMoreElements()){
			Association association=(Association)it.nextElement();
			if (association.getLinkType().compareTo(Association.CLASSE_ASSOCIATION)==0){
				if (association.getClassLinked().equals(linkedClasse)){
					classeSecondaire=UtilityClassFunction.secondaryClassForClassAss(classes, association);
					return classeSecondaire;
				}
			}
		}
		return classeSecondaire;
	}
	
	/**
	 * Cette mthode retourne la classe dite principale pour la classe lie passe en paramtre
	 * Lorsque deux classes sont associes l'une  l'autre  travers une classe association,
	 * Si la classe association commence par le nom d'une des classes, alors cette classe est dite principale.
	 * L'autre classe est dite secondaire.
	 * Si par exemple on a les classe Etudiant et Matiere qui sont lie par une classe association de nom EtudiantCoefficient alors Etudiant
	 * est la classe principale. L'autre est donc secondaire.
	 * Si par contre cette rgle n'est pas respecte une des classe devient classe principale (cela de faon arbitraire)
	 * La particularit de cette mthode, c'est que l'on ne connait pas l'association qui relie les classes. On la recherche grace  la classe association
	 * @param classes l'ensemble des classes
	 * @param association la classe association pour laquelle on recherche la classe secondaire
	 * @return retourne la classe dite secondaire dans une classe association.
	 */
	public static Classe principalClassForClassAssociation(Hashtable<String,Classe>  classes, Classe classe,Association association){
		Classe linkedClass=association.getClassLinked();
		Classe principalClass;
		Classe tempClass=(Classe)classes.get(association.getStart().getClassId());
		int index=linkedClass.getNom().indexOf(tempClass.getNom());
		if (index==0){
			principalClass=tempClass;
		}else{
			principalClass=(Classe)classes.get(association.getEnd().getClassId());
		}
		return principalClass;
	}
	
	/**
	 * Cette mthode retourne la classe dite secondaire dans une classe association.
	 * Lorsque deux classes sont associes l'une  l'autre  travers une classe association,
	 * Si la classe association commence par le nom d'une des classes, alors cette classe est dite principale.
	 * L'autre classe est dite secondaire.
	 * Si par exemple on a les classe Etudiant et Matiere qui sont lie par une classe association de nom EtudiantCoefficient alors Etudiant
	 * est la classe principale. L'autre est donc secondaire.
	 * Si par contre cette rgle n'est pas respecte une des classe devient classe principale (cela de faon arbitraire)
	 * @param classes l'ensemble des classes
	 * @param association la classe association pour laquelle on recherche la classe secondaire
	 * @return retourne la classe dite secondaire dans une classe association.
	 */
	public static Classe secondaryClassForClassAss(Hashtable<String,Classe>  classes,Association association){
		Classe linkedClass=association.getClassLinked();
		Classe secondaryClass;
		Classe tempClass=(Classe)classes.get(association.getStart().getClassId());
		int index=linkedClass.getNom().indexOf(tempClass.getNom());
		if (index==0){
			secondaryClass=(Classe)classes.get(association.getEnd().getClassId());
		}else{
			secondaryClass=tempClass;
		}
		return secondaryClass;
	}
	

	
	/**
	 * Cette mthode retourne vrai si la classe passe en paramtre est fille dans le classe de l'association 
	 * association. Si l'association n'est pas une association d'hritage, la classe ne peut tre fille pour cette association
	 * faux est donc retourne.
	 * @param classe la classe 
	 * @param association l'association 
	 * @return retourne vrai si la classe passe en paramtre est fille dans le classe de l'association 
	 */
	public static boolean child(Classe classe, Association association){
		if (association.getLinkType().compareTo(Association.HERITAGE)==0){
			if (classe.getId().compareTo(association.getEnd().getClassId())==0)
				return true;
			else
				return false;
		}
		return false;
	}
	
	
	/**
	 * Cette mthode retourne vraie si la Classe classe passe en paramtre est la classe mre dans l'association
	 * passe en parmtre. 
	 * l'association doit tre une association d'hritage. 
	 * @param classe la classe dont on veut voir si elle est la classe mre dans l'association
	 * @param association l'assocation
	 * @return retourne vraie si la Classe classe passe en paramtre est la classe mre dans l'association
	 * passe en parmtre. 
	 */
	public static boolean isMotherInAssociation(Classe classe, Association association){
		return (association.getStart().getClassId().compareTo(classe.getId())==0) && 
			(association.getLinkType().compareTo(Association.HERITAGE)==0);
}


	/**
	 * Cette mthode retourne la classe fille dans l'association passe ne parmtre
	 * @param classes la liste des classe
	 * @param association l'association ou l'on dterminer  la classe fille
	 * @return retourne la classe fille dans l'association passe ne parmtre
	 */
	public static Classe childClass(Hashtable<String,Classe>  classes,Association association){
		return (Classe)classes.get(association.getEnd().getClassId());
	}

	
	
	
	
}
